from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os
import commands

#BSRDGborder by blaksaga
#Based on RDGborder by ravydavygravy

class BSRDGborder ( Sensor ):

	def __init__ ( self, containercontentwidth = "50", containercontentheight = "50", containerresizable = "1" ):

		global _; _ = i18n.Translator ( "BSRDGborder" )

		Sensor.__init__ ( self )

		#These are the possible folders where themes can be found
		home = os.getenv ( "HOME" )
		self.__themefolders = [ home + "/.gdesklets/BSRDGThemes/", "/usr/share/gdesklets/Themes/BSRDGThemes/" ]

		#Theme values
		self._set_config_type ( "theme", TYPE_STRING, self.__getthemes () [0] [1] )

		#Container values
		self._set_config_type ( "containercontentwidth", TYPE_INT, containercontentwidth )
		self._set_config_type ( "containercontentheight", TYPE_INT, containercontentheight )
		self._set_config_type ( "containerhpadding", TYPE_INT, 0 )
		self._set_config_type ( "containervpadding", TYPE_INT, 0 )

		self._set_config_type ( "bghpadding", TYPE_INT, 0 )
		self._set_config_type ( "bgvpadding", TYPE_INT, 0 )

		self.__containerresizable = int ( containerresizable )

		#Border values
		self._set_config_type ( "bordershowright", TYPE_BOOL, 1 )
		self._set_config_type ( "bordershowleft", TYPE_BOOL, 1 )
		self._set_config_type ( "bordershowtop", TYPE_BOOL, 1 )
		self._set_config_type ( "bordershowbottom", TYPE_BOOL, 1 )
		self._set_config_type ( "borderthickness", TYPE_INT, 10 )

		self._watch_config ( self.__onconfigchange )

		self.add_timer ( 0, self.__updatedisplay )


	def get_configurator ( self ):

		configurator = self._new_configurator ()
		configurator.set_name ( _( "Border" ) )

		#Theme options
		configurator.add_title (_ ( "Border Theme" ) )

		configurator.add_option(_ ( "Theme" ), "theme", _( "Border theme to use" ), self.__getthemes () )

		#Container options
		configurator.add_title (_ ( "Container" ) )

		if ( self.__containerresizable == 1 ):
			configurator.add_spin (_ ( "Width" ), "containercontentwidth", _( "Width of the content container" ), 30, 500 )
			configurator.add_spin (_ ( "Height" ), "containercontentheight", _( "Height of the content container" ), 30, 500 )

		configurator.add_spin (_ ( "Horizontal padding:" ), "containerhpadding", _( "Pad the container x pixels of transparency" ), 0, 150 )
		configurator.add_spin (_ ( "Vertical padding:" ), "containervpadding", _( "Pad the container x pixels of transparency" ), 0, 150 )

		configurator.add_spin (_ ( "BG horizontal padding:" ), "bghpadding", _( "Pad the container with x pixels of background" ), 0, 150 )
		configurator.add_spin (_ ( "BG vertical padding:" ), "bgvpadding", _( "Pad the container with x pixels of background" ), 0, 150 )

		#Border options
		configurator.add_title (_ ( "Borders" ) )

		configurator.add_checkbox (_ ( "top" ), "bordershowtop", _( "Show the top border?" ) )
		configurator.add_checkbox (_ ( "bottom" ), "bordershowbottom", _( "Show the bottom border?" ) )
		configurator.add_checkbox (_ ( "left" ), "bordershowleft", _( "Show the left border?" ) )
		configurator.add_checkbox (_ ( "right" ), "bordershowright", _( "Show the right border?" ) )

		configurator.add_spin (_ ( "Thickness:" ), "borderthickness", _( "Thickness of the border in pixels" ), 5, 20 )

		return configurator


	def __updatedisplay ( self ):

		output = self.new_output ()

		#Get the config settings
		themedir = self.get_config ( "theme" )
		borderthickness = self.get_config ( "borderthickness" )
		bordershowtop = self.get_config ( "bordershowtop" )
		bordershowbottom = self.get_config ( "bordershowbottom" )
		bordershowleft = self.get_config ( "bordershowleft" )
		bordershowright = self.get_config ( "bordershowright" )
		containerhpadding = self.get_config ( "containerhpadding" )
		containervpadding = self.get_config ( "containervpadding" )
		bghpadding = self.get_config ( "bghpadding" )
		bgvpadding = self.get_config ( "bgvpadding" )
		containercontentwidth = self.get_config ( "containercontentwidth" )
		containercontentheight = self.get_config ( "containercontentheight" )

		#Set the theme images
		output.set ( "themetl", themedir + "tl.png" )
		output.set ( "themete", themedir + "te.png" )
		output.set ( "themetr", themedir + "tr.png" )
		output.set ( "themebl", themedir + "bl.png" )
		output.set ( "themebe", themedir + "be.png" )
		output.set ( "themebr", themedir + "br.png" )
		output.set ( "themele", themedir + "le.png" )
		output.set ( "themere", themedir + "re.png" )
		output.set ( "themebg", themedir + "bg.png" )

		#Set the border options
		borderthicknesst = borderthickness * int ( 0 or bordershowtop )
		borderthicknessb = borderthickness * int ( 0 or bordershowbottom )
		borderthicknessl = borderthickness * int ( 0 or bordershowleft )
		borderthicknessr = borderthickness * int ( 0 or bordershowright )

		output.set ( "borderthicknesst", borderthicknesst )
		output.set ( "borderthicknessb", borderthicknessb )
		output.set ( "borderthicknessl", borderthicknessl )
		output.set ( "borderthicknessr", borderthicknessr )

		#Show the main borders?
		output.set ( "bordershowleft", bordershowleft )
		output.set ( "bordershowright", bordershowright )
		output.set ( "bordershowtop", bordershowtop )
		output.set ( "bordershowbottom", bordershowbottom )

		#Show the border corners?
		output.set ( "bordershowtopleft", bordershowtop and bordershowleft )
		output.set ( "bordershowtopright", bordershowtop and bordershowright )
		output.set ( "bordershowbottomleft", bordershowbottom and bordershowleft )
		output.set ( "bordershowbottomright", bordershowbottom and bordershowright )

		#Container boundaries
		containerleft = borderthickness * bordershowleft
		containertop = borderthickness * bordershowtop
		containerwidth = containercontentwidth + ( self.get_config ( "containerhpadding" ) * 2 ) + ( self.get_config ( "bghpadding" ) * 2 )
		containerheight = self.get_config ( "containercontentheight" ) + ( self.get_config ( "containervpadding" ) * 2 ) + ( self.get_config ( "bgvpadding" ) * 2 )
		output.set ( "containerleft", containerleft )
		output.set ( "containertop", containertop )
		output.set ( "containerright", containerleft + containerwidth )
		output.set ( "containerbottom", containertop + containerheight )
		output.set ( "containerwidth", containerwidth )
		output.set ( "containerheight", containerheight )

		#Container content boudaries
		output.set ( "containercontentleft", containerleft + containerhpadding + bghpadding )
		output.set ( "containercontenttop", containertop + containervpadding + bgvpadding )
		output.set ( "containercontentwidth", containercontentwidth )
		output.set ( "containercontentheight", containercontentheight )

		#Border boundaries
		output.set ( "bgleft", containerleft + containerhpadding )
		output.set ( "bgtop", containertop + containervpadding )
		output.set ( "bgwidth", containerwidth - ( containerhpadding * 2 ) )
		output.set ( "bgheight", containerheight - ( containervpadding * 2 ) )

		self.send_output ( output )


	def __onconfigchange ( self, key, value ):

		#Just update the display
		self.__updatedisplay ()


	def __getthemes ( self ):

		#list to hold themes
		themes = []

		#Loop through possible folders and find themes
		for folder in self.__themefolders:

			#Make sure it exists
			if os.path.exists ( folder ):

				#get themes and append to list
				for f in os.listdir ( folder ):
					if os.path.isdir ( folder + f ):
						themes.append ( [ f , folder + f + "/" ] )

		if ( not themes ):
			print "NO THEMES FOUND"

		#return list of themes
		return themes


def new_sensor ( args ): return apply ( BSRDGborder, args )
