from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os



#
# Sensor for a binary clock.
#
class BinaryClock(Sensor):

    
    # This is attribut for the configuration of the desklet
    MODE_12H = "12h"
    MODE_24H = "24h"
    
    MODE_OCTAL       = 010
    MODE_DECIMAL     = 10
    MODE_HEXADECIMAL = 0x10

    THEME_DARK  = "dark"
    THEME_LIGHT = "light"
    BLUE_LED    = "blue"
    RED_LED     = "red"


    #
    #Initialisation of the sensor.
    #
    def __init__(self, theme):
        global _; _ = i18n.Translator("binaryclock-sensor")
        Sensor.__init__(self)
        
        self.__time = 0
        self.__theme     = "reset_theme"
        self.__led_theme = "reset_theme"
        self.__theme     = theme
        
	self._set_config_type("show-seconds", TYPE_BOOL,   1)
	self._set_config_type("mode",         TYPE_STRING, self.MODE_24H)
	self._set_config_type("binary_mode",  TYPE_INT,    self.MODE_DECIMAL)
	self._set_config_type("theme",        TYPE_STRING, self.THEME_DARK)
	self._set_config_type("led_theme",    TYPE_STRING, self.BLUE_LED)
	self.__path = os.getcwd()
        
	self.add_timer(0, self.__get_data)

        
    #
    # Returns the configuration GUI.
    #
    def get_configurator(self):

        configurator = self._new_configurator()
        
	configurator.set_name(_("BinaryClock"))
	configurator.add_title(_("Binary Clock Settings"))
	configurator.add_checkbox(_("_Show seconds"), "show-seconds",
                              _("Toggles the seconds on and off"))
	configurator.add_option(_("Display mode"), "mode",
                                _("12h or 24h display mode"),
                                [(_("12h"), self.MODE_12H),
                                 (_("24h"), self.MODE_24H)])
	configurator.add_option(_("Binary mode"), "binary_mode",
                                _("Octal, Decimal or Hexadecimal display"),
                                [(_("Octal"),       self.MODE_OCTAL),
                                 (_("Decimal"),     self.MODE_DECIMAL),
                                 (_("Hexadecimal"), self.MODE_HEXADECIMAL)])
	if self.__theme != "psi":
		configurator.add_option(_("Theme"), "theme",
                                _("Choose for a light or dark theme to fit your desktop"),
                                [(_("Dark"),  self.THEME_DARK),
                                 (_("Light"), self.THEME_LIGHT)])
	configurator.add_option(_("Theme for Led"), "led_theme",
                                _("Choose theme for the led"),
                                [(_("Blue Led"), self.BLUE_LED),
                                 (_("Red Led"),  self.RED_LED)])
	return configurator


    

    #
    # Method call each second to refresh the desklet
    #
    def __get_data(self):

        output = self.new_output()
	show_secs   = self.get_config("show-seconds")
	binary_mode = self.get_config("binary_mode")
	mode        = self.get_config("mode")
	theme       = self.get_config("theme")
	led_theme   = self.get_config("led_theme")

	if theme != self.__theme and self.__theme != "psi":
		output.set("theme_right",  os.path.join(self.__path, "icons", "theme", "right_%(theme)s.png" % vars()))
		output.set("theme_center", os.path.join(self.__path, "icons", "theme", "center_%(theme)s.png" % vars()))
		output.set("theme_left",   os.path.join(self.__path, "icons", "theme", "left_%(theme)s.png" % vars()))
		self.__theme = theme
                
	if led_theme != self.__led_theme:
            # redraw all
            self.__time = 0

        hour, minute, second = time.localtime()[3:6]

        tensec, sec = divmod(second, binary_mode)
        tenmin, min = divmod(minute, binary_mode)
        if mode == self.MODE_12H:
            hour %= 12
        tenhour, hour = divmod(hour, binary_mode)

        now = time.time()
        
        if now < self.__time or (now - self.__time) > 3:
            # time changed or something bad happened

            if show_secs:
                output.set("sec",    os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(sec)s.png" % vars()))
                output.set("tensec", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(tensec)s.png" % vars()))
            else:
                output.set("sec",    os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "blank.png"))
                output.set("tensec", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "blank.png"))

            output.set("min",     os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(min)s.png" % vars()))
            output.set("tenmin",  os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(tenmin)s.png" % vars()))
            output.set("hour",    os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(hour)s.png" % vars()))
            output.set("tenhour", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(tenhour)s.png" % vars()))

        else:
            # ok, just update what is needed
            if show_secs:
                output.set("sec", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(sec)s.png" % vars()))
                if sec == 0:
                    output.set("tensec", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(tensec)s.png" % vars()))
            else:
                output.set("sec",    os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "blank.png"))
                output.set("tensec", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "blank.png"))

            if tensec == 0 and sec == 0:
                output.set("min", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(min)s.png" % vars()))

                if min == 0:                           
                    output.set("tenmin", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(tenmin)s.png" % vars()))
                    
                    if tenmin == 0:
                        output.set("hour", os.path.join(self.__path, "icons","led", "%(led_theme)s" % vars(), "%(hour)s.png" % vars()))
                        
                        if hour == 0:                            
                            output.set("tenhour", os.path.join(self.__path, "icons", "led", "%(led_theme)s" % vars(), "%(tenhour)s.png" % vars()))
        # endif

	# Output the result on the desklet
        self.send_output(output)
	# Reset the timer.
        
        self.__time = now
        self.add_timer(1000, self.__get_data)



def new_sensor(args): return BinaryClock(*args)
