# coding=UTF-8

from transform import *

#
# Sensor for BubbleFishyMon
# Version 0.1 (16/04/2004)
#
# This module contains the Duck class, which handles movement and rendering of
# the duck.
#
# By Vassili Geronimos (mr_spot@optusnet.com.au)
#
# Based on the BubbleFishyMon dockapp
#


DUCK_GRAVITY = 1
DUCK_TERMINAL_VELOCITY = 5
DUCK_SPEED   = 0.5

DUCK_SUBMERGE_BY = 3 # pixels

DUCK_WIDTH = 32
DUCK_HEIGHT = 27

class Duck:
	
	# x,y co-ordinates of vertices of each polygon
	GFX_DUCK_OUTLINE = [0, 22, 1, 25, 3, 27, 27, 27, 27, 30, 31, 22, 32, 18, 32, 15, 31, 11, 29, 9, 28, 9, 26, 11, 23, 14, 19, 14, 19, 13, 21, 11, 22, 8, 22, 6, 20, 2, 16, 0, 13, 0, 10, 1, 7, 4, 4, 4, 2, 5, 2, 6, 7, 11, 7, 13, 1, 18, 0, 20]
	GFX_DUCK_BODY = [1, 22, 2,24, 4,26, 27,26, 29,24, 30,22, 31,18, 31,15, 30,11, 29,10, 28,10, 23,15, 18,15, 18,13, 20,11, 21,8, 21,6, 19,3, 16,1, 13,1, 10,2, 8,4, 8,7, 10,9, 9,10, 8,11, 8,14, 2,19, 1,20, 1,22] 
	GFX_DUCK_BILL = [ 3,5, 3,6, 7,10, 8,10, 9,9, 7,7, 7,5, 3,5] 
	GFX_DUCK_EYE  = [ 11,6,11,4,13,4,13,6 ]
	
	def __init__(self):
		# Start at the left heading right
		self.x = 0
		self.dir = 1
		
		# Note that for the duck, the y value is the position of the
		# BOTTOM of the duck.
		self.y = CANVAS_HEIGHT 
		
		# No initial y velocity
		self.dy = 0
	
	def update(self, water):
		# Has the duck moved off the display? If so, turn it around
		if self.x < -DUCK_WIDTH or self.x > CANVAS_WIDTH:
			self.dir = -self.dir

		waterlevel = water.minlevel(self.x,self.x + DUCK_WIDTH)
		
		# Is the duck below the waterline? If so, it needs to float up
		if self.y - DUCK_SUBMERGE_BY > waterlevel: 
			# If it's completely submerged, then move it closer to
			# the water level. A submerged duck is good to no one.
			if self.y > waterlevel + DUCK_HEIGHT:
				self.y = waterlevel + DUCK_HEIGHT
			
			self.dy = -1 # move upwards

		# Or is the duck above the waterline? If so, it needs to fall
		# down to it. 
		elif self.y - DUCK_SUBMERGE_BY < waterlevel:
			self.dy = self.dy  + DUCK_GRAVITY
			
		if self.dy < -DUCK_TERMINAL_VELOCITY: self.dy = -DUCK_TERMINAL_VELOCITY
		elif self.dy > DUCK_TERMINAL_VELOCITY: self.dy = DUCK_TERMINAL_VELOCITY

		self.y = self.y + self.dy # Move the duck
		self.x = self.x + self.dir * DUCK_SPEED

	def render(self):
		outline = self.GFX_DUCK_OUTLINE[:]
		body = self.GFX_DUCK_BODY[:]
		bill = self.GFX_DUCK_BILL[:]
		eye = self.GFX_DUCK_EYE[:]
		
		if self.dir == 1:
			outline = flip_x(outline,DUCK_WIDTH)
			body = flip_x(body,DUCK_WIDTH)
			bill = flip_x(bill,DUCK_WIDTH)
			eye = flip_x(eye,DUCK_WIDTH)
			
		ypos = self.y - DUCK_HEIGHT # Because self.y is the bottom of the duck
		
		gfx =        "color #000000," + transform_poly(outline, self.x, ypos, 1)
		gfx = gfx + ",color #f8cb3e," + transform_poly(body,    self.x, ypos, 1)
		gfx = gfx + ",color #c93912," + transform_poly(bill,    self.x, ypos, 1)
		gfx = gfx + ",color #000000," + transform_poly(eye,     self.x, ypos, 1)
		
		return gfx
