# coding=UTF-8

from random import *

import gtk

from transform import *

#
# Sensor for BubbleFishyMon
# Version 0.1 (16/04/2004)
#
# This module contains the Fishy class, which handles moving and rendering a
# single fishy.
#
# By Vassili Geronimos (mr_spot@optusnet.com.au)
#
# Based on the BubbleFishyMon dockapp
#

MIN_FISH_SPEED = 5
MAX_FISH_SPEED = 20

class Fishy:
	GFX_FISH_OUTLINE = [1, 8, 3, 11, 9, 13, 11, 13, 12, 14, 14, 14, 15, 13, 14, 12, 15, 11, 18, 9, 23, 14, 22, 12, 23, 10, 22, 8, 23, 7, 22, 5, 23, 3, 18, 8, 14, 5, 12, 4, 13, 2, 10, 1, 8, 2, 7, 4, 3, 5]
	
	GFX_FISH_FILL = [2, 8, 4, 10, 9, 12, 12, 12, 12, 13, 13, 13, 14, 12, 13, 11, 14, 10, 17, 8.5, 13, 6, 12, 5, 8, 5, 4, 6]
	
	GFX_FISH_TAIL = [19, 9, 21, 11, 22, 10, 21, 8.5, 22, 7, 21, 6, 19, 8]
	
	GFX_FISH_FIN = [8, 5, 12, 5, 12, 3, 10, 2, 9, 3]
	
	GFX_FISH_GILLS = [[6, 7, 6, 8, 5, 9], [7, 6, 8, 8, 7, 10], [9, 6, 10, 8, 9, 11], [11, 6, 12, 8, 11, 11]]
	
	GFX_FISH_EYE = [4, 7, 4, 9, 6, 9, 6, 7]
	
	def __init__(self, y, vel, dir, colour):
		self.y = y
		if vel < MIN_FISH_SPEED: vel = MIN_FISH_SPEED
		if vel > MAX_FISH_SPEED: vel = MAX_FISH_SPEED
		self.vel = vel
		self.dir = dir # 1 if moving right, -1 if moving left
		
		
		if dir == -1: self.x = CANVAS_WIDTH
		else: self.x = -24
		
		self.col = colour

	def accel(self, amt):
		
		# Adjust amt so that it doesn't make the fishy go too fast or
		# too slow
		if (self.vel + amt) <= MIN_FISH_SPEED: amt = self.vel - MIN_FISH_SPEED
		if (self.vel + amt) >  MAX_FISH_SPEED: amt = MAX_FISH_SPEED - self.vel
		
		self.vel = self.vel + amt
		
		# Tell caller what adjustment was made.
		return amt 

	def is_offscreen(self):
		if self.x < -24 or self.x > (CANVAS_WIDTH + 24):
			return 1
		else:
			return 0
	
	# water should be an instance of the “Water” class
	def update(self, water):
		self.x = self.x + self.dir * self.vel
		self.y = self.y + uniform(-2,2)

		# Do comparisons in this order, so that fish will always be
		# below waterline, but only below bottom edge of canvas if
		# waterline makes this necessary.
		if self.y > CANVAS_HEIGHT-8: self.y = CANVAS_HEIGHT-8
		if self.y < water.minlevel(self.x,self.x + 24): self.y = water.minlevel(self.x, self.x + 24)

	def render(self):
		outline = self.GFX_FISH_OUTLINE[:]
		fill = self.GFX_FISH_FILL[:]
		tail = self.GFX_FISH_TAIL[:]
		fin = self.GFX_FISH_FIN[:]
		eye = self.GFX_FISH_EYE[:]
		gills = []	
		for g in self.GFX_FISH_GILLS[:]:
			gills.append(g[:])
		
		if self.dir == 1:
			outline = flip_x(outline,24)
			fill = flip_x(fill,24)
			tail = flip_x(tail,24)
			fin = flip_x(fin,24)
			eye = flip_x(eye,24)
			for g in gills: g = flip_x(g,24)
			
		col = gtk.gdk.color_parse(self.col)
		bodycol = self.col
		fincol = "#%02x%02x%02x" % (col.red * 0.75 / 256, col.green * 0.75 / 256, col.blue * 0.75 / 256)
		gillcol = "#%02x%02x%02x" % (col.red * 1.5 / 256, col.green * 1.5 / 256, col.blue * 1.5 / 256)
		
		gfx = "color black,  " + transform_poly(outline, self.x, self.y, 1)
		gfx = gfx + ",color " + bodycol + "," + transform_poly(fill,    self.x, self.y, 1)
		gfx = gfx + "," + transform_poly(tail, self.x, self.y, 1)
		gfx = gfx + ",color %s," % fincol + transform_poly(fin, self.x, self.y, 1)
		for g in gills: 
			gfx = gfx + ",color %s," % gillcol + transform_pline(g, self.x, self.y)
		gfx = gfx + ",color black," + transform_poly(eye, self.x, self.y, 1)
		
		return gfx

	def score(self):
		return min(0, self.vel - MIN_FISH_SPEED)
