# coding=UTF-8

from random import *
from transform import *
from colour import *

#
# Sensor for BubbleFishyMon
# Version 0.1 (16/04/2004)
#
# This module contains the Water class, which handles the waterlevel and
# bubbles. 
#
# By Vassili Geronimos (mr_spot@optusnet.com.au)
#
# Based on the BubbleFishyMon dockapp
#


# Uhh... fix when i know what a good value is.
WATER_SPEED_LIMIT = 2
WATER_VISCOSITY   = 0.98
WATER_VOLATIVITY  = 1.00

class Water:
	def __init__(self):
		self.points   = [CANVAS_HEIGHT for x in range(CANVAS_WIDTH)]
		self.d_points = [0 for x in range(CANVAS_WIDTH)]
		self.height = CANVAS_HEIGHT
		self.bubbles = []

	# Most of this water code is copied from bubblefishymon. 
	def update(self):
		self.points[0] = self.height
		self.points[CANVAS_WIDTH-1] = self.height
		
		# Do 4 passes of this update code, otherwise the water moves too slow
		for i in range(5):
			# The endpoints are only controlled by the memory level, so don't update them.
			for n in range(1, CANVAS_WIDTH - 1):
				# Find change in velocity for point n
				dv = self.points[n-1] + self.points[n+1] - 2 * self.points[n]
				dv = dv * WATER_VOLATIVITY
				
				self.d_points[n] = (self.d_points[n] + dv) * WATER_VISCOSITY
				
				if self.d_points[n] >  WATER_SPEED_LIMIT: 
					self.d_points[n] =  WATER_SPEED_LIMIT
				if self.d_points[n] < -WATER_SPEED_LIMIT: 
					self.d_points[n] = -WATER_SPEED_LIMIT
	
			for n in range(1, CANVAS_WIDTH - 1):	
				self.points[n] = self.points[n] + self.d_points[n]
	
				if self.points[n] >= CANVAS_HEIGHT: 
					self.points[n] = CANVAS_HEIGHT
				if self.points[n] <= 0: 
					self.points[n] = 0
				
		for bubble in self.bubbles[:]:
			bubble.update()
			x1 = max(min(CANVAS_WIDTH-1,int(bubble.x)),0)
			x2 = max(min(CANVAS_WIDTH-1,int(bubble.x + 3)),0)
		
			if bubble.y <= self.minlevel(x1,x2):
				# Pop bubble
				self.bubbles.remove(bubble)
				for x in range(x1,x2):
					self.points[x] = self.points[x] + 3

	# Yes, I am aware than min and max are around the other way for these functions.

	def minlevel(self, x1, x2):
		x1 = max(min(CANVAS_WIDTH-1,int(x1)),0)
		x2 = max(min(CANVAS_WIDTH-1,int(x2)),0)

		if x1 == x2: return self.points[x1]
		
		return max(self.points[x1:x2])

	def maxlevel(self, x1, x2):
		x1 = max(min(CANVAS_WIDTH-1,int(x1)),0)
		x2 = max(min(CANVAS_WIDTH-1,int(x2)),0)

		if x1 == x2: return self.points[x1]
		
		return min(self.points[x1:x2])

	def create_bubble(self):
		bubble = Bubble()
		
		x1 = max(min(CANVAS_WIDTH-1,int(bubble.x)),0)
		x2 = max(min(CANVAS_WIDTH-1,int(bubble.x + 3)),0)
		
		for x in range(x1,x2):
			self.points[x] = self.points[x] - 3
		
		self.bubbles.append(bubble)

	def render(self, colour):
		poly_coords = []
		for x in range(CANVAS_WIDTH):
			poly_coords.append(x)
			poly_coords.append(self.points[x])

		poly_coords[len(poly_coords):] = [CANVAS_WIDTH,self.height,CANVAS_WIDTH,CANVAS_HEIGHT,0,CANVAS_HEIGHT]
		
		gfx = "color %s, " % colour + transform_poly(poly_coords, 0, 0, 1)

		bubblecolour = colour_trans(colour, "#ffffff", 0.8)
		for bubble in self.bubbles:
			gfx = gfx + "," + bubble.render(bubblecolour)

		return gfx



# Define the Bubble class here, since it is so trivial and no one else actually
# needs to use it.
		
BUBBLE_GRAVITY = 0.2

class Bubble:	
	def __init__(self):
		self.x = uniform(1, CANVAS_WIDTH-1)
		self.y = CANVAS_HEIGHT
		self.v = 0

	def update(self):
		self.v = self.v - BUBBLE_GRAVITY
		self.y = self.y + self.v

	def render(self,colour):
		return "color %s, " % colour + transform_poly([0,0,0,3,3,3,3,0], self.x, self.y, 1)
