from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import dialog

from Icon import Icon
from NewButtonConfig import NewButtonConfig
from EditButtonConfig import EditButtonConfig
from NewFolderConfig import NewFolderConfig
from EditFolderConfig import EditFolderConfig

import os
import commands
import math
import gtk

_ANIMATE_STOP = 0
_ANIMATE_ENTER = 1
_ANIMATE_LEAVE = 2
_ANIMATE_CLICK = 3
_ANIMATE_CLICK_BACK = 4
_ANIMATE_TO_STOP = 5

_FRAME_TIME = 25
_CLICK_SPEED = 0.2
_CLICK_WAIT = -3.0

class config:
	length = 1
	folder_length = 1
	size = 32
	folder_size = 24
	scale = 1.3
	folder_scale = 1.0
	saturation = 0.3
	radius = 168
	msg = ""
	title = ""
	icons = []
	folder_icons = []
	circle_home = ""
	buttons_file = ""
	in_icon = -1
	in_folder = -1
	active_folder = 0
	show_title = True
	show_folders = True

class CircleButtonBar(Sensor):

	_circle_home = ""
	_animation_semaphore = 1

	def __init__(self):

		Sensor.__init__(self)

		self._set_config_type("radius", TYPE_INT, 160)
		self._set_config_type("isize", TYPE_INT, 32)
		self._set_config_type("fsize", TYPE_INT, 24)
		self._set_config_type("iscale", TYPE_FLOAT, 2.0)
		self._set_config_type("show_title", TYPE_BOOL, True)
		self._set_config_type("show_folders", TYPE_BOOL, True)
		self._set_config_type("show_middle_image", TYPE_BOOL, True)
		self._set_config_type("disable_animations", TYPE_BOOL, False)
		self._set_config_type("bg_visible", TYPE_BOOL, True)
		self._set_config_type("bg_file", TYPE_STRING, "gfx/bg/bg.png")
		self._watch_config(self.__on_watch_config)

		config.circle_home = os.path.expanduser("~/.circlebuttonbar")
		self._circle_home = config.circle_home
		config.folders_file = os.path.join(config.circle_home, "folders")
		if(not os.path.exists(config.circle_home)):
			os.mkdir(config.circle_home)

		if(not os.path.exists(config.folders_file)):
			fd = open(config.folders_file, "w")
			fd.write("applications.folder\tgfx/icons/gnome-logo-icon-transparent.png\tApplications\n")
			fd.close()
			fd = open(os.path.join(config.circle_home, "applications.folder"), "w")
			fd.write("gnome-terminal\tgfx/icons/gnome-terminal.png\tTerminal\n")
			fd.write("xmms\tgfx/icons/xmms.xpm\tXMMS\n")
			fd.write("mozilla\tgfx/icons/mozilla-browser.png\tMozilla\n")
			fd.write("xchat\tgfx/icons/xchat.png\tXChat\n")
			fd.close()

		fd = open(config.folders_file, "r")
		lines = fd.readlines()
		fd.close()

		if(len(lines) > 0):
			for i in xrange(len(lines)):
				config.folder_icons.append(Icon())
				tokens = str.split(str.strip(lines[i]), "\t")
				config.folder_icons[i].command = tokens[0]
				config.folder_icons[i].icon = tokens[1]
				config.folder_icons[i].name = tokens[2]
				config.folder_icons[i].iconx = config.folder_icons[i].icony = 0
				config.folder_icons[i].animate = _ANIMATE_STOP
				config.folder_icons[i].animation_state = 1.0
				config.folder_icons[i].init_pos_x = config.folder_icons[i].init_pos_y = 1.0

		self._load_active_folder()

		self._add_timer(0, self.__init_data)
		self._add_timer(_FRAME_TIME, self.__animate)

	def _load_active_folder(self):
		if(len(config.folder_icons[config.active_folder].children) == 0):
			fd = open(os.path.join(config.circle_home,
				config.folder_icons[config.active_folder].command), "r")
			lines = fd.readlines()
			fd.close()

			icons = []

			if(len(lines) > 0):
				for i in xrange(len(lines)):
					icons.append(Icon())
					tokens = str.split(str.strip(lines[i]), "\t")
					icons[i].command = tokens[0]
					icons[i].icon = tokens[1]
					icons[i].name = tokens[2]
					icons[i].iconx = icons[i].icony = 0
					icons[i].animate = _ANIMATE_STOP
					icons[i].animation_state = 1.0
					icons[i].init_pos_x = icons[i].init_pos_y = 1.0

			config.folder_icons[config.active_folder].children = icons
		config.icons = config.folder_icons[config.active_folder].children
		output = self._new_output()
		output.set("title", config.folder_icons[config.active_folder].name)
		output.set("title-icon", config.folder_icons[config.active_folder].icon)
		self._send_output(output)

	def _add_new_icon(self, icon):
		config.icons.append(icon)

	def _add_new_folder(self, icon):
		config.folder_icons.append(icon)

	def _circle_pos(self, step):
		an = (2.0 * math.pi / config.length) * step
		ax = (-(math.sin(an)/2.0)+0.5)*0.9 + 0.05
		ay = (-(math.cos(an)/2.0)+0.5)*0.9 + 0.05
		return ax, ay

	def _small_circle_pos(self, state):
		an = (2.0 * math.pi * state)
		ax = math.sin(an)/2.0
		ay = math.cos(an)/2.0
		return ax, ay

	def _generate_folders_menu(self, excluding):
		menu = []
		for i in xrange(config.folder_length):
			if(i != excluding):
				menu.append((config.folder_icons[i].name, True, None,
							self.__move_folder_after, [excluding, i]))
		return menu

	def _interpolate(self, a, b, i):
		return a - (a - b) * float(i)

	def __init_data(self):
		config.radius = self._get_config("radius")
		config.size = self._get_config("isize")
		config.folder_size = self._get_config("fsize")
		config.scale = self._get_config("iscale")
		config.length = len(config.icons)
		config.folder_length = len(config.folder_icons)
		config.show_title = self._get_config("show_title")
		config.show_folders = self._get_config("show_folders")
		for i in xrange(config.length):
			config.icons[i].iconx, config.icons[i].icony = self._circle_pos(i)

		output = self._new_output()

		for i in xrange(config.length):
			output.set("size[%d]" % i, config.size)
			output.set("folder-size[%d]" % i, config.folder_size)
			output.set("alignx[%d]" % i, config.icons[i].iconx)
			output.set("aligny[%d]" % i, config.icons[i].icony)
			output.set("iradius[%d]" % i, config.radius)
			output.set("fradius[%d]" % i, config.folder_size * config.folder_scale)
			output.set("scale[%d]" % i, 1.0)
			if(i != config.active_folder):
				output.set("folder-scale[%d]" % i, 1.0)
			else:
				output.set("folder-scale[%d]" % i, config.folder_scale)
			output.set("saturation[%d]" % i, config.saturation)
			output.set("folder-saturation[%d]" % i, config.saturation)
			output.set("iconuri[%d]" % i, config.icons[i].icon)

		for i in xrange(config.folder_length):
			output.set("folder-iconuri[%d]" % i, config.folder_icons[i].icon)
			
		output.set("radius", config.radius)
		output.set("layout", "vertical, " + str(0))
		output.set("length", config.length)
		output.set("folders-length", config.folder_length)
		output.set("textx", config.radius / 2 - 1)
		output.set("texty", config.radius / 2 - 1)
		output.set("msg", config.msg)
		output.set("title", config.folder_icons[config.active_folder].name)
		output.set("title-icon", config.folder_icons[config.active_folder].icon)
		output.set("show-title", config.show_title)
		output.set("show-folders", config.show_folders)
		output.set("bg-uri", self._get_config("bg_file"))
		output.set("bg-visible", self._get_config("bg_visible"))
		self._send_output(output)

		return 0

	def __animate(self):
		if(self._animation_semaphore > 1):
			self._animation_semaphore -= 1
			return False

		for i in xrange(config.length):
			scale_state = 0.0
			if(config.icons[i].animate == _ANIMATE_ENTER):
				cx, cy = self._circle_pos(i)
				sx, sy = self._small_circle_pos(config.icons[i].animation_state)
				config.icons[i].iconx = cx + sx * 0.05
				config.icons[i].icony = cy + sy * 0.05
				config.icons[i].animation_state += 0.05
				if(config.icons[i].animation_state > 1.0):
					config.icons[i].animation_state = 0.0
			elif(config.icons[i].animate == _ANIMATE_LEAVE):
				cx, cy = self._circle_pos(i)
				config.icons[i].iconx = self._interpolate(config.icons[i].init_pos_x, cx, config.icons[i].animation_state)
				config.icons[i].icony = self._interpolate(config.icons[i].init_pos_y, cy, config.icons[i].animation_state)
				config.icons[i].animation_state += 0.2
				if(config.icons[i].animation_state > 1.0):
					config.icons[i].animate = _ANIMATE_STOP
			elif(config.icons[i].animate == _ANIMATE_CLICK):
				cx, cy = self._circle_pos(i)
				config.icons[i].animation_state += _CLICK_SPEED
				scale_state = config.icons[i].animation_state
				if(config.icons[i].animation_state > 1.0):
					config.icons[i].animation_state = 1.0
				config.icons[i].iconx = self._interpolate(cx, 0.5, config.icons[i].animation_state)
				config.icons[i].icony = self._interpolate(cy, 0.5, config.icons[i].animation_state)
				if(config.icons[i].animation_state >= 1.0):
					config.icons[i].animation_state = _CLICK_WAIT
					config.icons[i].animate = _ANIMATE_CLICK_BACK
			elif(config.icons[i].animate == _ANIMATE_CLICK_BACK):
				cx, cy = self._circle_pos(i)
				config.icons[i].animation_state += _CLICK_SPEED
				if(config.icons[i].animation_state < 0):
					scale_state = 1.0
				if(config.icons[i].animation_state > 1.0):
					config.icons[i].animation_state = 1.0
				if(config.icons[i].animation_state > 0):
					scale_state = 1 - config.icons[i].animation_state
					config.icons[i].iconx = self._interpolate(0.5, cx, config.icons[i].animation_state)
					config.icons[i].icony = self._interpolate(0.5, cy, config.icons[i].animation_state)
				if(config.icons[i].animation_state >= 1.0):
					config.icons[i].animate = _ANIMATE_TO_STOP

			if(config.icons[i].animate != _ANIMATE_STOP):
				output = self._new_output()
				output.set("scale[%d]" % i, self._interpolate(1.0, config.scale, scale_state))
				output.set("alignx[%d]" % i, config.icons[i].iconx)
				output.set("aligny[%d]" % i, config.icons[i].icony)
				self._send_output(output)

			if(config.icons[i].animate == _ANIMATE_TO_STOP):
				config.icons[i].animate = _ANIMATE_STOP
	
		if(self._get_config("disable_animations")):
			self._animation_semaphore = 0
			return False
		else:
			return True

	def call_action(self, action, path, args=[]):

		if(action=="enter-icon"):
			index=path[-1]
			config.in_icon = index
			config.in_folder = -1
			if(config.icons[index].animate == _ANIMATE_STOP):
				config.icons[index].animate = _ANIMATE_ENTER
				config.icons[index].animation_state = 0.0
			output = self._new_output()
			output.set("msg", config.icons[index].name)
			output.set("saturation[%d]" % index, 1.0)
			output.set("center-img-uri", config.icons[index].icon)
			output.set("center-img-visible", self._get_config("show_middle_image") and True)
			self._send_output(output)
		elif(action=="leave-icon"):
			index=path[-1]
			if(config.in_icon == index):
				config.in_icon = -1
			config.icons[index].animate = _ANIMATE_LEAVE
			config.icons[index].init_pos_x = config.icons[index].iconx
			config.icons[index].init_pos_y = config.icons[index].icony
			config.icons[index].animation_state = 0.0
			output = self._new_output()
			if((config.in_icon == -1) and (config.in_folder == -1)): 
				output.set("msg", "")
				output.set("center-img-visible", False)
			output.set("saturation[%d]" % index, config.saturation)
			self._send_output(output)
		elif(action=="click-icon"):
			index=path[-1]
			config.icons[index].animate = _ANIMATE_CLICK
			config.icons[index].animation_state = 0.0
			output = self._new_output()
			os.system("cd ~ && " + config.icons[index].command + " & disown")
			self._send_output(output)
		elif(action=="enter-folder"):
			index=path[-1]
			config.in_folder = index
			config.in_icon = -1
			output = self._new_output()
			output.set("msg", config.folder_icons[index].name)
			output.set("folder-saturation[%d]" % index, 1.0)
			output.set("center-img-uri", config.folder_icons[index].icon)
			output.set("center-img-visible", self._get_config("show_middle_image") and True)
			self._send_output(output)
		elif(action=="leave-folder"):
			index=path[-1]
			if(config.in_folder == index):
				config.in_folder = -1
			output = self._new_output()
			if(config.in_icon == -1 and config.in_folder == -1): 
				output.set("msg", "")
				output.set("center-img-visible", False)
			output.set("folder-saturation[%d]" % index, config.saturation)
			self._send_output(output)
		elif(action=="click-folder"):
			index=path[-1]
			if(config.active_folder != index):
				config.active_folder = index
				self._load_active_folder()
				self.__init_data()
		elif(action=="menu"):
			removable = (config.in_icon != -1)
			removable_folder = (config.in_folder != -1)
			move_in_folders = (len(config.folder_icons) > 1)
			menu = []

			menu.append(("New Button", True, None, self.__new_button, []))
			menu.append(("New Folder", True, None, self.__new_folder, []))
			if(removable):
				menu.append(())
				menu.append(("Edit Button", True, None, self.__edit_button,
											[config.in_icon]))
				menu.append(("Remove Button", True, None, self.__remove_button,
											[config.in_icon]))
			if(removable_folder):
				menu.append(())
				menu.append(("Edit Folder", True, None, self.__edit_folder,
											[config.in_folder]))
				menu.append(("Remove Folder", move_in_folders, None,
							self.__remove_folder, [config.in_folder]))
#				if(move_in_folders):
#					menu.append(())
#					submenu = self._generate_folders_menu(config.in_folder)
#					menu.append(("Move Folder After", True, submenu, None, []))
			if(move_in_folders):
				menu.append(())
				menu.append(("Next Folder", move_in_folders, None,
											self.__next_folder, []))
				menu.append(("Previous Folder", move_in_folders, None,
											self.__prev_folder, []))
			self._open_menu(menu)

	def _notify_icon_change(self):
		self.__init_data()
		self.__save_button_file()

	def _notify_folder_change(self):
		self.__save_folder_file()
		self._load_active_folder()
		self.__init_data()

	def __new_button(self, args):
		nbc = NewButtonConfig(self)

	def __new_folder(self, args):
		nbc = NewFolderConfig(self)

	def __edit_button(self, args, icon):
		nbc = EditButtonConfig(self, config.icons[icon])

	def __remove_button(self, args, icon):
		config.icons.remove(config.icons[icon])
		self._notify_icon_change()

	def __edit_folder(self, args, icon):
		nbc = EditFolderConfig(self, config.folder_icons[icon])

	def __remove_folder(self, args, icon):
		if(config.active_folder == icon):
			config.active_folder = 0
			self._load_active_folder()
		config.folder_icons.remove(config.folder_icons[icon])
		self.__init_data()
		self.__save_folder_file()
		self._load_active_folder()

	def __prev_folder(self, args):
		config.active_folder -= 1
		if(config.active_folder < 0):
			config.active_folder = len(config.folder_icons) - 1
		self._load_active_folder()
		self.__init_data()

	def __next_folder(self, args):
		config.active_folder += 1
		if(config.active_folder == len(config.folder_icons)):
			config.active_folder = 0
		self._load_active_folder()
		self.__init_data()

	def __save_folder_file(self):
		fd = open(config.folders_file, "w")
		for i in xrange(config.folder_length):
			fd.write(config.folder_icons[i].command + "\t")
			fd.write(config.folder_icons[i].icon + "\t")
			fd.write(config.folder_icons[i].name + "\n")
		fd.close()

	def __save_button_file(self):
		fd = open(os.path.join(config.circle_home,
			config.folder_icons[config.active_folder].command), "w")
		for i in xrange(config.length):
			fd.write(config.icons[i].command + "\t")
			fd.write(config.icons[i].icon + "\t")
			fd.write(config.icons[i].name + "\n")
		fd.close()	

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name("Circle ButtonBar")
		configurator.add_title("Button Settings")
		configurator.add_spin("Radius: ", "radius", "radius of the ButtonBar", 80, 240)
		configurator.add_spin("Button Size: ", "isize", "size of button icons", 8, 96)
		configurator.add_spin("Folder Size: ", "fsize", "size of folder icons", 8, 96)
		configurator.add_spin("Icon Scale: ", "iscale", "scale of icon when clicked", 0.1, 5.0) 
		configurator.add_checkbox("Show Background", "bg_visible", "show background image") 
		configurator.add_checkbox("Show Title", "show_title", "show title of active folder") 
		configurator.add_checkbox("Show Folders", "show_folders", "show folders") 
		configurator.add_checkbox("Show Highlighted Button", "show_middle_image", "show highlighted button in the middle") 
		configurator.add_checkbox("Disable Animations", "disable_animations", "disable button animations") 
		configurator.add_file_selector("Background Image", "bg_file", "optional background image");

		configurator.add_close_callback(self._configurator_close_callback)
		return configurator

	def __on_watch_config(self, key, value):
		if(key == "radius"):
			config.radius = value
			output = self._new_output()
			output.set("radius", config.radius)
			for i in xrange(config.length):
				output.set("iradius[%d]" % i, config.radius)
			self._send_output(output)
		elif(key == "isize"):
			config.size = value
			output = self._new_output()
			for i in xrange(config.length):
				output.set("size[%d]" % i, config.size)
			output.set("size", config.size)
			self._send_output(output)
		elif(key == "fsize"):
			config.folder_size = value
			output = self._new_output()
			for i in xrange(config.length):
				output.set("folder-size[%d]" % i, config.folder_size)
			self._send_output(output)
		elif(key == "iscale"):
			config.scale = value
			output = self._new_output()
			output.set("iscale", config.scale)
			self._send_output(output)
		elif(key == "show_title"):
			config.show_title = value
			output = self._new_output()
			output.set("show-title", config.show_title)
			self._send_output(output)
		elif(key == "show_folders"):
			config.show_folders = value
			output = self._new_output()
			output.set("show-folders", config.show_folders)
			self._send_output(output)
		elif(key == "bg_visible"):
			output = self._new_output()
			output.set("bg-visible", value)
			self._send_output(output)
		elif(key == "disable_animations"):
			if(value == "false"):
				self._animation_semaphore += 1
				self._add_timer(_FRAME_TIME, self.__animate)
			

	def _configurator_close_callback(self):
		output = self._new_output()
		output.set("bg-uri", self._get_config("bg_file"))
		output.set("bg-visible", self._get_config("bg_visible"))
		self._send_output(output)

def new_sensor(args): return apply(CircleButtonBar, args)
