from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os

from DiskIoSensorFactory import DiskIoSensorFactory, DiskIoSensor

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value

class DiskIo(Sensor):

	def __init__(self, interval = "1000"):
		
		global _; _ = i18n.Translator("disk-io-sensor")
		
		Sensor.__init__(self)
		
		self.__read = 0
		self.__write = 0
		self.__speed_read = 0
		self.__speed_write = 0
		
		self.__sensor = DiskIoSensorFactory.createSensor()
		assert isinstance(self.__sensor, DiskIoSensor)
		
		self._set_config_type("interval", TYPE_INT, int(interval))
		self._set_config_type("formatted_output", TYPE_BOOL, 1)
		self._add_timer(int(interval), self.__on_tick)
		self._add_thread(self.__command_thread)

	def __on_tick(self):
		data = self._new_output()
		
		if self._get_config("formatted_output"):
			data.set("read", human_readable(self.__read)+"B")
			data.set("write", human_readable(self.__write)+"B")
		else:
			data.set("read", str(self.__read) + "B")
			data.set("write", str(self.__write) + "B")
			
		data.set("speed_read", human_readable(self.__speed_read)+"B/s")
		data.set("speed_write", human_readable(self.__speed_write)+"B/s")
		data.set("speed_total", human_readable(self.__speed_write + self.__speed_read)+"B/s")
		
		data.set("speed_total_plot", self.__speed_write + self.__speed_read)
		
		self._send_output(data)
		self._add_timer(self._get_config("interval"), self.__on_tick)
		return 0

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Disk IO"))
		configurator.add_title(_("Display Configuration"))
		configurator.add_spin(_("Update every (ms):"), "interval",
					_("Update every x milliseconds"),
					100, 60000)
		configurator.add_checkbox(_("Formatted output"), "formatted_output",
					_("Enable human readable format (ie. KB, MB, GB)"))
		return configurator
	
	def __command_thread(self):
		old_read, old_write = self.__get_read_write_bytes()
		while (1):
			if (self._is_stopped()): break

			interval = self._get_config("interval") / 1000.0
			self.__read, self.__write = self.__get_read_write_bytes()
			
			self.__speed_read = (self.__read - old_read) / interval
			self.__speed_write = (self.__write - old_write) / interval

			old_read, old_write = self.__read, self.__write
			time.sleep(interval)

	def __get_read_write_bytes(self):
		return self.__sensor.get_read_write_bytes()
		
def new_sensor(args): return apply(DiskIo, args)
