#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
    Modul:          __init__
    Description:    Beschreibung
    Version:        V0.1
    Copyright:      2003 by Fritz Cizmarov fritz@sol.at
    Last modified:  03. Okt. 2003
    License:        free
    Exports:        Classes and Functions to export
"""

import re, httplib
from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n 

_ = i18n.Translator("ExternIP")

class ExternIP(Sensor):
    ip_check_url    = "checkip.dyndns.org"
    interval        = 3600
    font            = "Sans 12"
    fcolor          = "Black"
    ip              = "-N/A-"
    ip_regex        = '([\d]{1,3}\.[\d]{1,3}\.[\d]{1,3}\.[\d]{1,3})'
    def __init__(self):
        Sensor.__init__(self)
        self._set_config_type("ip_check_url", TYPE_STRING, self.ip_check_url)
        self._set_config_type("interval", TYPE_INT, self.interval)
        self._set_config_type("font", TYPE_STRING, self.font)
        self._set_config_type("fcolor", TYPE_STRING, self.fcolor)
        self.to_send = []
        self.cnt = 0
        self._add_timer(1000, self.do_timer)

    def get_configurator(self):
        self._set_config("ip_check_url", self.ip_check_url)
        cfg = self._new_configurator()
        cfg.set_name(_("ExternIP"))
        cfg.add_title(_("ExternIP Desklet\nSettings"))
        cfg.add_entry(_("url"), "ip_check_url",  _("URL to get my IP from"))
        cfg.add_spin(_("Update every (s):"), "interval",
                     _("Updates the display every x seconds"),
                     1, 86400)
        cfg.add_font_selector(_("Font:"), "font", 
                              _("The font which is used to display the labels"))
        cfg.add_color_selector(_("Text color:"), "fcolor",
                               _("The Color of the labels"))
        return cfg
        
    def __update_config(self):
        ip_check_url = self._get_config("ip_check_url")
        interval = self._get_config("interval")
        font = self._get_config("font")
        fcolor = self._get_config("fcolor")
        if ip_check_url != self.ip_check_url:
            self.ip_check_url = ip_check_url
            self.cnt = 0
        if interval != self.interval:
            self.interval = interval
            self.cnt = 0
        if font != self.font:
            self.font = font
            self.to_send.append(("font", font))
        if fcolor != self.fcolor:
            self.fcolor = fcolor
            self.to_send.append(("fcolor", fcolor))

    def do_timer(self):
        self.to_send = []
        if self.cnt <= 0:
            self.cnt = self.interval
            try:
                urlsplit = re.split('(http://)?([^/]*)(.*)', self.ip_check_url)
                host = urlsplit[2]
                path = urlsplit[3]
                connection = httplib.HTTPConnection(host)
                if connection:
                    connection.request("GET", path)
                    res = connection.getresponse()
                    if res.reason == "OK":
                        ip = re.split(self.ip_regex, res.read())[1]
                    else:
                        ip = res.reason
                    connection.close()
            except:
                ip = "NO CONNECTION"
            if self.ip != ip:
                self.ip = ip
                self.to_send.append(("ip", ip))
        self.__update_config()
        if self.to_send:
            output = self._new_output()
            for to_send in self.to_send:
                output.set(to_send[0],to_send[1])
            self._send_output(output)
        self.cnt -= 1
        return 1

def new_sensor(args): return apply(ExternIP, args)

