from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

from random import choice

import string
import os
import time
import commands

 
class Hottie ( Sensor ):

	def __init__ ( self ):

		global _; _ = i18n.Translator ( "Hottie-Sensor" )

		#The lastimage string so we don't use an image twice in a row
		self.__lastimage = ""

		#Initialize the sensor and define config types
		Sensor.__init__ ( self )
		self._set_config_type ( "imageFolder", TYPE_STRING, "/usr/share/gdesklets/Displays/Hottie-0.2.2/gfx/females" )
		self._set_config_type ( "imageDim", TYPE_INT, 0 )
		self._set_config_type ( "showCaption", TYPE_BOOL, 1 )
		self._set_config_type ( "imageCaption", TYPE_STRING, "Hottie of the Hour" )
		self._set_config_type ( "captop", TYPE_INT, "0" )

		#Load an image
		self.add_timer ( 2000, self.__changeimage, 1 )

		#If the config changes, update the display
		self.watch_config ( self.__config_changed )


	#Configuration options
	def get_configurator ( self ):

		configurator = self._new_configurator ()

		configurator.set_name (_ ( "Hottie" ) )

		#Image options
		configurator.add_title (_ ( "Images" ) )
		configurator.add_entry (_ ( "Image folder: " ), "imageFolder", _( "In what folder are the hotties located?" ) )
		configurator.add_spin (_ ( "Dim image: " ), "imageDim", _( "Dim the image x levels to make it look better on dark desktops." ), 0, 3 )

		#Caption options
		configurator.add_title (_ ( "Caption" ) )
		configurator.add_checkbox (_ ( "Show caption: " ), "showCaption", _( "Whether or not the caption below the image should be shown." ) )
		configurator.add_entry (_ ( "Caption:" ), "imageCaption", _( "The caption to be shown under the image" ) )
		configurator.add_spin (_ ( "Caption Position: " ), "captop", _( "Vertical Position of the Caption" ), 0, 500 )

		return configurator


	#Change the hottie image
	# settimer is 0 or 1...do we want to set another timer
	# we only want to set it if the old timer has expired
	def __changeimage ( self, settimer ):
		output = self.new_output ()

		#get the user defined image folder
		imageFolder = self.get_config ( "imageFolder" )

		#cut the / off the end if it exists
		imageFolder = string.rstrip ( imageFolder , "/" )

		#the types of images we will allow
		use_types = ['.jpg', '.gif', '.png']

		#list to hold all images that can be found
		imgs = []

		#if it is a valid folder
		if os.path.exists ( imageFolder ) and os.path.isdir ( imageFolder ):

			#get all files in that folder
			for f in os.listdir ( imageFolder ):

				#splitext may fail
				try:
					#if the file is of proper type then add it to the list
					if os.path.splitext ( f ) [ 1 ] in use_types:
						imgs.append ( imageFolder + os.sep + f )
				except:
					pass

		image = self.__lastimage

		#if we found images, choose a random one
		if ( imgs ):
			#don't repeat the same image
			while ( image == self.__lastimage ):
				image = choice ( imgs )
				#if there is only one image, don't go into an infinite loop :)
				if ( len ( imgs ) == 1 ):
					break
			print "Changing image to " + image
		#otherwise, use the default image
		else:
			print "No image found in folder " + imageFolder
			image = "gfx/default.png"

		#set the last image so images don't repeat
		self.__lastimage = image

		#set the display image
		output.set ( "image", image )
		self.send_output ( output )
		self.__lastimage = image

		#set the dimness and caption
		self.__config_changed ( "imageDim", self.get_config ( "imageDim" ) )
		self.__config_changed (	"showCaption", self.get_config ( "showCaption" ) )
		self.__config_changed ( "imageCaption", self.get_config ( "imageCaption" ) )
		self.__config_changed ( "captop", self.get_config ( "captop" ) )

		#reset the timer to one hour
		if ( settimer == 1 ):
			self.add_timer ( 1000 * 60 * 60, self.__changeimage, 1 )


	#Configuration changes
	def __config_changed ( self, key, value ):

		#Change the dimness
		if ( key == "imageDim" ):
			#Output the dimness to the display
			output = self.new_output ()
			output.set ( "imageDim", value )
			self.send_output ( output )

		#Change the folder
		if ( key == "imageFolder" ):
			#all we have to do is call __changeimage :)
			self.__changeimage ( 0 )

		#Toggle the caption
		if ( key == "showCaption" ):
			#Output the caption boolean to the display
			output = self.new_output ()
			output.set ( "showCaption", value )
			self.send_output ( output )

		#Change the caption text
		if ( key == "imageCaption" ):
			#Output the caption to the display
			output = self.new_output ()
			output.set ( "imageCaption", value )
			self.send_output ( output )

		#Change the caption position
		if ( key == "captop" ):
			#Output the value to the display
			output = self.new_output ()
			output.set ( "captop", value )
			self.send_output ( output )


	#Action performed
	def call_action ( self, call, path, args = [] ):

		#The right-click menu
		if call == "open_menu":
			self.open_menu ( [ (_ ( "C_hange image" ), 1, None, self.__changeimage, [] ) ] )

		#Drag and drop a folder
		if call == "drag_n_drop":
			#get the directory of the dropped folder
			folder = args [ 0 ] [ 0 ]

			#strip the file:// from the beginning
			folder = "/" + string.lstrip ( folder , "file:/" )

			#set the new folder
			self.set_config ( "imageFolder", folder )


def new_sensor ( args ): return apply ( Hottie, args )
