from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import iCon
import time

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value

class ImoncSensor(Sensor):

	def __init__(self, hostname = "fli4l", port = 5000, pw = "", interval = "1000"):
		self.con = None
		global _; _ = i18n.Translator("imonc-sensor")

		self._hostname = ""
		self._version = ""
		self._uptime = ""
		self._date = ""
		self._cpu = 0
		self._inout = [0,0]
		
		# call the constructor of the super class
		Sensor.__init__(self)

		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("imond_host", TYPE_STRING, hostname)
		self._set_config_type("imond_port", TYPE_INT, port)
		self._set_config_type("imond_pass", TYPE_STRING, pw)

		self._add_timer(int(interval), self.__my_timer)
		self._add_thread(self.__command_thread)

	def __my_timer(self):
		if (self.con == None or self.con.connected == 0):
			# connect to router
			self.con = iCon.ImondConnection(self._get_config("imond_host"),self._get_config("imond_port"),self._get_config("imond_pass"))
			self.con.connect()

		if (self.con.connected == 0):
			return 1

		output = self._new_output()
		output.set("hostname", self._hostname)
		output.set("version", self._version)
		output.set("uptime", self._uptime)
		output.set("date", self._date)
		output.set("cpu", self._cpu)

		output.set("in", human_readable(self._inout[0]))
		output.set("out", human_readable(self._inout[1]))
		output.set("inout", self._inout[0]+self._inout[1])
		self._send_output(output)
		self._add_timer(self._get_config("update"), self.__my_timer)

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name("Imonc")
		configurator.add_title(_("Imonc Configuration"))
		configurator.add_entry(_("Hostname"), "imond_host", _("Remote host name"))
		configurator.add_spin(_("Port"), "imond_port", _("Remote port adress"),0,65536)
		configurator.add_entry(_("Password"), "imond_pass", _("Remote imond password"))

		configurator.add_spin(_("Update every (ms):"), "update",
					_("Updates imond info every x milliseconds"),
					100, 60000)
		return configurator

	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
					
			self._hostname = self.con.get_hostname()
			self._version = self.con.get_version()
			self._uptime = self.con.get_uptime()
			self._date = self.con.get_date()
			self._cpu = self.con.get_cpu()
	
			self._inout = self.con.get_rate()
			time.sleep(self._get_config("update")*0.001)

	def _shutdown(self):
		if (self.con != None):
			self.con.disconnect()
		print "disconnected"

def new_sensor(args): return apply(ImoncSensor, args)
