##
## SETI@Home sensor for gDesklets
## Author: Alen Bicskei <husky at mad hu>
## Created: 10 Sept. 2003
## License: GNU/GPL
##

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import time
import os
import commands
import fpformat
import exceptions

def ticktoclock(tick):
    h = int(tick/3600)
    m = int((tick-h*3600)/60)
    s = int((tick-h*3600-m*60))
    if m<10:m0="0"
    else:m0=""
    if s<10:s0="0"
    else:s0=""
    return str(h)+":"+m0+str(m)+":"+s0+str(s)

class LTVSeti(Sensor):

	
	def __my_timer(self):
	    data = self._new_output()
	    if self.__cpu > 0: offset = time.time()-self.__updated
	    else: offset = 0
	    data.set("prog", int(self.__prog*100))
	    data.set("nresults", int(self.__nresults))
	    data.set("cpu", int(self.__cpu))
	    data.set("status", fpformat.fix(float(self.__prog*100),2) + "%")
	    data.set("elapsed", ticktoclock(self.__cpu + offset))
	    if self.__cpu*100/(self.__prog*100)-self.__cpu - (offset) > 0:
		data.set("remaining", ticktoclock( self.__cpu*100/(self.__prog*100)-self.__cpu - (offset) ))
	    else:
		data.set("remaining", "N/A")
	    self._send_output(data)
            return 1
	
	def __init__(self, interval = "30000", path = "/usr/seti"):
		global _; _ = i18n.Translator("setiathome")
		Sensor.__init__(self)
		self.__excepted=1
		self.__nresults=0
		self.__cpu=0
		self.__prog=0.000001
		self.__updated=time.time()
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("seti_path", TYPE_STRING, path)
		self._add_timer(1000, self.__my_timer)
		self._add_thread(self.__command_thread)

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("SETI@Home"))
		configurator.add_title(_("Display Configuration"))
		configurator.add_spin(_("Update every (ms):"), "update", _("Updates the display"), 1, 600000)
		configurator.add_entry(_("SETI@Home path:"), "seti_path",  _("Path to SETI@Home data."))
		return configurator
	
	def __command_thread(self):
		while (1):
			if (self._is_stopped()): break
			self.__excepted=2
			
			try:
			    fd = open(self._get_config("seti_path") + "/state.sah", "r")
			except IOError,err:
			    self.__excepted=1
			    self.__nresults=0
			    self.__cpu=0
			    self.__prog=0.000001

			if self.__excepted==2:
			    data = fd.read()
			    fd.close()
			    data = data.splitlines()
			
			    for line in data :
				fields = line.split()
				if int(fields[0].find("prog=")) == 0: self.__prog=float(fields[0].replace("prog=", ""))
				if int(fields[0].find("cpu=")) == 0: self.__cpu=float(fields[0].replace("cpu=", ""))

			    fd = open(self._get_config("seti_path") + "/user_info.sah", "r")
                    	    data = fd.read()
			    fd.close()
			    data = data.splitlines()

			    for line in data :
				fields = line.split()
				if int(fields[0].find("nresults=")) == 0: self.__nresults=float(fields[0].replace("nresults=", ""))

			self.__updated=time.time()
			time.sleep(self._get_config("update")*0.001)

def new_sensor(args): return apply(LTVSeti, args)
