"""
__init__.py
"""

try:
	True, False
except:
	True, False = 1, 0

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n, dialog
from xml.sax.saxutils import escape

import os
from stat import *
import Queue
import time

import screenbuffer
import watchlist

has_py_wrapper = False
try:
	import textwrap
	has_py_wrapper = True
except:
	pass

WRAP_NONE = 0
WRAP_CROP = 1
WRAP_LABEL = 2
WRAP_TEXTWRAP = 3
UPDATE_NORMAL = 0
UPDATE_SLOW = 1
UPDATE_FAST = 2
UPDATE_DIRECT = 3

__author__ = "Bas van Oostveen"
__name__ = "MultiTail"
__version__ = "0.3.1"
__license__ = "GPL"

class DefaultConfig:
	""" default config """
	length = 6
	scroll_length = 50
	scroll_jump = True
	update_speed = 0
	enable_intro = True
	enable_logname = False
	enable_linecount = False
	enable_showerror = True
	enable_dropshadow = False
	dropshadow_offset = 1
	background_theme = "light"
	font = "Courier New 14"
	display_width = 80
	display_grow = True
	wraptype = 0
	watches = ["/var/log/messages"]
	names = ["messages"]
	colors = ["#000000"]
	states = ["ok"]
	_watches = "/var/log/messages"
	_names = "messages"
	_colors = "#000000"
	_states = "ok"

class MultiTail(Sensor):
	def __init__(self, *args):
		""" init """
		global _; _ = i18n.Translator("MultiTail")
		
		# vars
		self.__force_buffer_update = False
		#self.__queue = Queue.Queue(64)
		self.__background_themes = ["transparent","dark","light"]
		self.__background_prefix = "gfx/"
		self.__background_layout = [
			"bg.png",			# left center
			"bg.png",			# top center
			"bg.png",			# right center
			"bg.png",			# bottom center
			"top-left.png",		# top left
			"top-right.png",	# top right
			"bottom-right.png",	# bottom right
			"bottom-left.png"	# bottom left
		]		
		
		# init superclass
		Sensor.__init__(self)
		
		# set config types
		self._set_config_type("length", TYPE_INT, DefaultConfig.length)
		self._set_config_type("scroll_length", TYPE_INT, DefaultConfig.scroll_length)
		self._set_config_type("scroll_jump", TYPE_BOOL, DefaultConfig.scroll_jump)
		self._set_config_type("update_speed", TYPE_INT, DefaultConfig.update_speed)
		self._set_config_type("enable_intro", TYPE_BOOL, DefaultConfig.enable_intro)
		self._set_config_type("enable_logname", TYPE_BOOL, DefaultConfig.enable_logname)
		self._set_config_type("enable_dropshadow", TYPE_BOOL, DefaultConfig.enable_dropshadow)
		self._set_config_type("enable_linecount", TYPE_BOOL, DefaultConfig.enable_linecount)
		self._set_config_type("enable_showerror", TYPE_BOOL, DefaultConfig.enable_showerror)
		self._set_config_type("dropshadow_offset", TYPE_INT, DefaultConfig.dropshadow_offset)
		self._set_config_type("background_theme", TYPE_STRING, DefaultConfig.background_theme)
		self._set_config_type("font", TYPE_STRING, DefaultConfig.font)
		self._set_config_type("display_width", TYPE_INT, DefaultConfig.display_width)
		self._set_config_type("display_grow", TYPE_BOOL, DefaultConfig.display_grow)
		self._set_config_type("wraptype", TYPE_INT, DefaultConfig.wraptype)
		self._set_config_type("watches", TYPE_LIST, DefaultConfig._watches)
		self._set_config_type("names", TYPE_LIST, DefaultConfig._names)
		self._set_config_type("colors", TYPE_LIST, DefaultConfig._colors)
		self._set_config_type("states", TYPE_LIST, DefaultConfig._states)
		
		# init screenbuffer
		self.__buffer = screenbuffer.Buffer(self)
		
		# register
		self._add_timer(0, self.__applet_init)
		self._add_timer(1000, self.__applet_update, True)
		self._add_thread(self.__command_thread)
		self._watch_config(self.__config_changed)
	
	def __check_config(self):
		wa = self._get_config("watches")
		na = self._get_config("names")
		co = self._get_config("colors")
		st = self._get_config("states")
		lwa, lna, lco, lst = len(wa), len(na), len(co), len(st)
		if not lwa==lna==lco==lst:
			print "Error loading log config, falling back to default."
			self._set_config("watches", DefaultConfig.watches)
			self._set_config("names", DefaultConfig.names)
			self._set_config("colors", DefaultConfig.colors)
			self._set_config("states", DefaultConfig.states)
		del wa,na,co,st,lwa,lna,lco,lst
	
	def __get_theme_layout(self, theme):
		p = self.__background_prefix
		
		if theme in ("trans", "transparent"):
			return p+(",%s"%p).join(["trans.png"]*8)
			
		if theme not in self.__background_themes:
			theme = DefaultConfig.background_theme
		
		p = p + theme+'/'
		l = self.__background_layout
		return p+(',%s'%p).join(l)
	
	def __get_theme_width(self, theme):
		if theme in ("trans", "transparent"):
			return "0,0,0,0"
		else:
			return "5,5,5,5"
	
	def __get_theme_background(self, theme):
		if theme in ("trans", "transparent"):
			return ""
			
		if theme not in self.__background_themes:
			theme = DefaultConfig.background_theme
		return self.__background_prefix+theme+'/bg.png'	
	
	def __setup_logarray(self, output, length=None, wraptype=None):
		length = length or self._get_config("length")
		wraptype = wraptype or self._get_config("wraptype")
		for c in xrange(length):
			for key in ("font", "enable_dropshadow", "dropshadow_offset"):
				value = self._get_config(key)
				output.set("%s[%d]" % (key, c), value)
			if wraptype==WRAP_LABEL:
				output.set("%s[%d]" % (key, c), value)
				
		return output
	
	def __setup_background(self, output, theme):
		output.set("frame_uris", self.__get_theme_layout(theme))
		output.set("frame_width", self.__get_theme_width(theme))
		output.set("bg_img", self.__get_theme_background(theme))
		return output
		
	def __setup_display_width(self, output):
		display_width = self._get_config("display_width")
		display_grow = self._get_config("display_grow")
		wraptype = self._get_config("wraptype")
		length = self._get_config("length")
		if display_grow==True:
			output.set("char_width", "")
			output.set("pixel_width", 0)
		else:
			if wraptype==WRAP_LABEL:
				output.set("char_width", "")
				output.set("pixel_width", display_width)
				for c in xrange(length):
					output.set("wrapat[%d]" % c, display_width)
			else:
				if self._get_config("enable_linecount"):
					display_width = display_width + 1
				if self._get_config("enable_logname"):
					names = self._get_config("names")
					longestFunc = lambda x,y: len(x)>len(y) and x or y
					longestName = reduce(longestFunc, names)
					bias = 2 # opening/closing brackets
					display_width = display_width + len(longestName) + bias
				output.set("char_width", " "*display_width)
				output.set("pixel_width", 0)
		return output
		
	def __config_changed(self, key, value):
		""" config changed event """
		# for these keys:
		
		# update the display
		if key in ("length"):
			output = self._new_output()
			output.set(key, value)
			output = self.__setup_logarray(output)
			self._send_output(output)
			self.__force_buffer_update = True
			return
		
		# update display width and grow
		if key in ("display_width", "display_grow"):
			output = self._new_output()
			output = self.__setup_display_width(output)
			self._send_output(output)
			return
		
		# update wraptype
		if key in ("wraptype"):
			# clear buffer
			self.__buffer.clear()
			if self._get_config("enable_intro"):
				self.__buffer.showIntro()
			# update display width
			output = self._new_output()
			output = self.__setup_display_width(output)
			self._send_output(output)
			return
		
		# update the background
		if key in ("background_theme"):
			output = self._new_output()
			output = self.__setup_background(output, value)
			self._send_output(output)
			return
		
		# update the logarray
		if key in ("font", "enable_dropshadow", "dropshadow_offset"):
			output = self._new_output()
			if key=="font":
				output.set("cwfont", value)
			for c in xrange(self._get_config("length")):
				output.set("%s[%d]" % (key, c), value)
			self._send_output(output)
			return
		
		# force buffer update
		if key in ("enable_logname"):
			self.__force_buffer_update = True
		
	def __applet_init(self):
		self.__check_config()
		# check python2.3> textwrapping
		wraptype = self.get_config("wraptype")
		if wraptype==WRAP_TEXTWRAP and not has_py_wrapper:
			wraptype = 0
			self.set_config("wraptype", 0)
			print "[multitail] cannot wrap, no module textwrap (py23>) found"

		# check theme
		if self._get_config("background_theme") not in self.__background_themes:
			# if theme not matched, fallback to default
			self._set_config("background_theme", DefaultConfig.background_theme)
		
		# show intro
		if self._get_config("enable_intro"):
			self.__buffer.showIntro()
		
		# init display
		output = self._new_output()
		
		# setup display
		for key in ["length", "display_width"]:
			output.set(key, self._get_config(key))
		
		# setup logarray
		output = self.__setup_logarray(output)
		
		# setup display wdith
		output = self.__setup_display_width(output)
		output.set("cwfont", self._get_config("font"))
		
		# setup background
		theme = self._get_config("background_theme")
		output = self.__setup_background(output, theme)
		
		# apply to display
		self._send_output(output)
		
		# never again
		return 0
		
	def __applet_update(self, schedule_next=False):
		""" update applet """
		length = self._get_config("length")
		
		output = self._new_output()
		output.set("length", length)
		
		if self._get_config("enable_linecount"):
			for c in xrange(length):
				output.set("linecount[%d]" % c, c)
		else:
			for c in xrange(length):
				output.set("linecount[%d]" % c, '')
		
		if self.__buffer.isChanged() or self.__force_buffer_update:
			self.__force_buffer_update = False
			c = 0
			for color, name, line in self.__buffer.list(length):
				if self._get_config("enable_logname"):
					output.set("name[%d]" % c, name)
				else:
					output.set("name[%d]" % c, "")
				output.set("line[%d]" % c, line)
				output.set("linecolor[%d]" % c, color)
				c=c+1
		
		self._send_output(output)
			
		# schedule next update
		if schedule_next:
			tick = 1000
			update_speed = self._get_config("update_speed")
			if update_speed==UPDATE_SLOW:
				# SLOW
				tick = tick*2
			elif update_speed==UPDATE_FAST:
				# FAST
				tick = tick/2
			self._add_timer(tick, self.__applet_update, True)
		return False
	
	def __command_thread(self):
		def add_to_buffer(c, n, l):
			# escape line and remove all line breaks
			l = escape(l.strip().replace('\n','').replace('\r',''))
			# apply wrapping (if set)
			wraptype = self._get_config("wraptype")
			if wraptype>0:
				display_width = self._get_config("display_width")
				if wraptype==WRAP_CROP:
					l = l[:display_width]
				elif wraptype==WRAP_TEXTWRAP and has_py_wrapper:
					l = textwrap.wrap(l, display_width, subsequent_indent="...")
			# add to buffer
			if isinstance(l, list):
				for line in l:
					self.__buffer.put(c, "[%s]" % n, line)
			else:
				self.__buffer.put(c, "[%s]" % n, l)
		
		tailbook = {}
		prev_states = []
		prev_watches = []
		prev_colors = []
		while (1):			
			if (self._is_stopped()):
				break
			
			# update tails in tailbook
			update_speed = self._get_config("update_speed")
			new_states = self._get_config("states")
			new_watches = self._get_config("watches")
			new_names = self._get_config("names")
			new_colors = self._get_config("colors")
			if (prev_watches!=new_watches) or (prev_colors!=new_colors):
				prev_watches = new_watches
				prev_colors = new_colors
				for r in [k for k in tailbook.keys() if k not in new_watches]:
					del tailbook[r]
				for i in range(len(new_watches)):
					k = new_watches[i]
					if not tailbook.has_key(k):
						tailbook[k] = {
							'name': new_names[i],
							'mtime': 0,
							'esize': 0
						}

			for i in range(len(new_watches)):
				if not new_states[i]=="ok":
					continue
				tail = new_watches[i]
				file = None
				try:
					stat = os.stat(tail)
					name = tailbook[tail]['name']
					size = stat[ST_SIZE]
					if size==0:
						# empty file
						continue
					# try to open file
					file = open(tail, "rb")
					esize = tailbook[tail]['esize']
					if size==esize:
						continue
					elif esize==0 or size<esize:
						# first read (backread one line) or file shrunk
						buff = ""
						tailbook[tail]['esize'] = size
						pos = stat[ST_SIZE]
						while (1):
							delim = buff.rfind(os.linesep)
							if delim <> -1:
								# found delimiter
								add_to_buffer(new_colors[i], name, buff[delim:])
								break
							pos = pos - 64
							if pos<0:
								pos = 0
							file.seek(pos)
							buff = file.read(64).strip() + buff
							if pos==0:
								add_to_buffer(new_colors[i], name, buff)
								break
					else:
						file.seek(esize)
						if update_speed==UPDATE_DIRECT:
							# read all new lines
							for newline in file.readlines():
								if newline:
									add_to_buffer(new_colors[i], name, newline)
						else:
							# read only one line
							newline = file.readline()
							if newline:
								add_to_buffer(new_colors[i], name, newline)
						tailbook[tail]['esize'] = file.tell()
				except (OSError, IOError), e:
					err = "%s: '%s'" % (e.strerror, e.filename)
					new_states[i] = err
					self._set_config("states", new_states)
					if self._get_config("enable_showerror"):
						add_to_buffer(new_colors[i], name, err)
					continue
			# close file
			if not file==None:
				file.close()
			# wait 
			tick = 1
			if update_speed==UPDATE_SLOW:
				# slow (2 seconds)
				tick = tick*2
			elif update_speed==UPDATE_FAST:
				# FAST (1/2 second)
				tick = tick/2.0
			time.sleep(tick)
	
	def get_configurator(self):
		""" create configurator """
		c = self._new_configurator()
		c.set_name(_(__name__))
		c.add_title(_("%s version %s" % (__name__,__version__)))
		
		c.add_title(_("Display (changes will only effect new lines)"))
		c.add_spin(_("Text/Display Width"), "display_width", _("Set in pixel for wrap-at mode or characters other textwrap modes"), 0, 4096)
		c.add_spin(_("Lines on screen:"), "length",
					_("Number of lines to display on screen"), 1, 127)
		c.add_option(_("Scroll speed:"), "update_speed", _("Set update speed"), [("normal",0),("slow",1),("fast",2),("direct",3)])
		wrap_options = [("None",0),("Crop",1),("Label Wrap-At",2)]
		if has_py_wrapper:
			wrap_options.append(("Textwrap (py23>)",3))
		c.add_option(_("Textwrap"), "wraptype", _("Select wrapping type"), wrap_options)
		c.add_checkbox(_("Grow/Shrink"), "display_grow", _("If enabled sizes are dynamic (fit-to-need)"))
		c.add_checkbox(_("Show intro"), "enable_intro", _("Show Intro"))
		c.add_checkbox(_("Show log filename"), "enable_logname", _("Display the filename before each line"))
		c.add_checkbox(_("Show linecount"), "enable_linecount", _("Display line numbers"))
		c.add_checkbox(_("Show errors"), "enable_showerror", _("Show IO-Errors on display"))
		
		c.add_title(_("ScrollBack"))
		c.add_spin(_("Remember # lines:"), "scroll_length",
					_("How many lines to remember for scrollback (0 = disabled)"), 0, 999)
		c.add_checkbox(_("Jump to new line"),"scroll_jump",_("Disable scrolling when new line appears."))
		
		c.add_title(_("Background"))
		c.add_option(_("Background theme"), "background_theme",
			    _("Background Themes"), [[theme]*2 for theme in self.__background_themes])
		
		c.add_title(_("Font"))
		c.add_font_selector(_("Font to use"), "font", _("The font to use."))
		c.add_checkbox(_("Enable dropshadow"), "enable_dropshadow", _("Enable dropshadow"))
		c.add_spin(_("Dropshadow offset"), "dropshadow_offset",
			    _("Offset of the dropshadow text\nfor if you want to make your multitail look fuzzy\nor if you got very strange fonts :)"), 1, 15)
		return c
	
	def __action_watch_dialog(self, src, *args):
		if (watchlist.win==None):
			watchlist.show(self._get_config, self._set_config)
		else:
			dialog.warning("Already open", "Watch list already open")
	
	def __action_clear_buffer(self, src, *args):
		self.__buffer.clear()
		if self._get_config("enable_intro"):
			self.__buffer.showIntro()
		self.__force_buffer_update=True
	
	def call_action(self, action, path, args=[]):
		""" react to user events/actions """
		# open menu
		if (action == "menu"):
			self._open_menu([
				("Tail files", 1, None, self.__action_watch_dialog, []),
				("Clear buffer", 1, None, self.__action_clear_buffer, [])
			])
		elif (action == "scroll"):
			direction = args[0]
			if direction==0:
				self.__buffer.scroll_up()
			elif direction==1:
				self.__buffer.scroll_down()
			self.__applet_update()
	
	def _shutdown(self):
		pass

def new_sensor(args): return apply(MultiTail, args)
