"""
ScreenBuffer
"""

try:
	True, False
except:
	True, False = 1, 0

import os
import copy
from xml.sax.saxutils import escape

EMPTY_TUPLE = ('','','')
INTRO_BUFFER = [
	('#101011','',"               __ __   _  __         _  __"),
	('#101000','',"  __ _  __ __ / // /_ (_)/ /_ ___ _ (_)/ /"),
	('#111011',''," /  ' \/ // // // __// // __// _ `// // / "),
	('#101011','',"/_/_/_/\_,_//_/ \__//_/ \__/ \_,_//_//_/  ")
]

# check for user defined intro message
h, f, ns, msg, color = "","",{},"",""
try:
	try:
		h = os.path.expanduser("~")
		f = os.path.join(h, ".gdesklets/multitail")
		execfile(f, ns)
		color = ns.has_key('COLOR') and ns['COLOR'] or "#000000"
		lines = ns['MESSAGE'].split("\n")
		INTRO_BUFFER = [(color,'',escape(l)) for l in lines if l]
	except:
		pass
finally:
	del h, f, ns, msg, color

class Buffer:
	def __init__(self, sensor):
		self.__sensor = sensor
		self.__buffer = []
		self.__offset = 0
		self.__longest_name = 0
		self.__changed = True
	
	def __slide(self):
		bl = self.__sensor._get_config("scroll_length")+self.__sensor._get_config("length")
		if bl==0:
			self.__buffer = []
		elif len(self.__buffer)>bl:
			self.__buffer = self.__buffer[-int(bl):]

	def showIntro(self):
		self.__buffer.extend(INTRO_BUFFER)
		self.__changed = True
		
	def scroll_up(self):
		if self.__offset+self.__sensor._get_config("length")>=len(self.__buffer):
			return
		self.__offset = self.__offset + 1
		self.__changed = True
	
	def scroll_down(self):
		if self.__offset>0:
			self.__offset = self.__offset - 1
			self.__changed = True
		else:
			self.__offset = 0

	def getSize(self):
		return len(self.__buffer)
	
	def isChanged(self):
		return self.__changed
	
	def put(self, color, name, line):
		self.__buffer.append((color, name, line))
		self.__slide()
		self.__changed = True
		if self.__sensor._get_config("scroll_jump"):
			self.__offset = 0
		elif not self.__offset==0:
			self.scroll_up()
	
	def list(self, length):
		self.__changed = False
		if self.__buffer==[]:
			return [EMPTY_TUPLE]*length
		
		s = len(self.__buffer)
		startAt = s-length-self.__offset
		if startAt<0:
			return self.__buffer[:]+([EMPTY_TUPLE]*-startAt)
		else:
			endAt = s-self.__offset
			return self.__buffer[startAt:endAt]
	
	def clear(self):
		self.__buffer = []
		self.__changed = True
