"""
listdialog.py
"""

import gtk
import gnome.ui
from utils import dialog
import os.path

win=None

class WatchList(gtk.Window):
	def __destroy(self, *args):
		global win
		self.destroy()
		win=None
		
	def __init__(self, get_config, set_config):
		self.watches = get_config("watches")
		self.names = get_config("names")
		self.colors = get_config("colors")
		self.states = get_config("states")
		
		assert len(self.watches)==len(self.colors)==len(self.states)==len(self.names)
		self.__set_config = set_config
	
		gtk.Window.__init__(self, gtk.WINDOW_TOPLEVEL)
		self.set_default_size(378, 196)
		self.set_title("Watch logs...")
		self.set_border_width(5)
		self.connect("destroy", self.__destroy)
		
		vbox = gtk.VBox(gtk.FALSE, 0)
		self.add(vbox)
		
		label = gtk.Label("Watch these logs (double click to change name or color)")
		vbox.pack_start(label, 0, 0)
		
		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		vbox.pack_start(sw)
		
		self.__selected_row = -1
		self.__columns = 4	# file | name | color | error
		self.list = gtk.CList(self.__columns)
		self.list.column_titles_show()
		self.list.set_column_title(0, "file")
		self.list.set_column_title(1, "name")
		self.list.set_column_title(2, "color")
		self.list.set_column_title(3, "state")
		self.list.set_column_width(0, 164)
		self.list.set_column_width(1, 64)
		self.list.set_column_width(2, 64)
		self.list.connect("select_row", self.__list_row_selected)
		sw.add(self.list)
		
		hbox = gtk.HBox(gtk.TRUE, gtk.FALSE)
		vbox.pack_start(hbox, gtk.FALSE, gtk.FALSE)
		b_add = gtk.Button("Add")
		b_add.connect("clicked", self.__add_file)
		hbox.pack_start(b_add, gtk.FALSE, gtk.TRUE)
		
		b_reset = gtk.Button("Reset")
		b_reset.connect("clicked", self.__reset_file)
		hbox.pack_start(b_reset, gtk.FALSE, gtk.TRUE)
		
		b_remove = gtk.Button("Remove")
		b_remove.connect("clicked", self.__remove_file)
		hbox.pack_start(b_remove, gtk.FALSE, gtk.TRUE)
		
		b_close = gtk.Button("Close")
		b_close.connect("clicked", self.__destroy)
		vbox.pack_start(b_close, gtk.FALSE, gtk.FALSE)
		
		self.__build_list()
		
		self.show_all()
	
	def __do_config(self):
		self.__set_config("watches", self.watches)
		self.__set_config("names", self.names)
		self.__set_config("colors", self.colors)
		self.__set_config("states", self.states)
	
	def __build_list(self):
		self.list.freeze()
		self.list.clear()
		for w, n, c, e in map(lambda w,x,y,z: (w,x,y,z),
							self.watches, self.names, self.colors, self.states):
			entree = self.list.append(' ' * self.__columns)
			self.list.set_text(entree, 0, w)
			self.list.set_text(entree, 1, n)
			self.list.set_text(entree, 2, c)
			self.list.set_text(entree, 3, e)
		self.list.thaw()

	def __list_row_selected(self, widget, row, col, event):
		self.__selected_row = row
		if (event.type==gtk.gdk._2BUTTON_PRESS):
			if (col==1):
				self.__change_name(row)
			elif (col==2):
				self.__change_color(row)
	
	def __change_name(self, row):
		c0 = self.list.get_text(row, 0)
		try:
			c0i = self.watches.index(c0)
			old_name = self.names[c0i]
		except:
			print "[MultiTail] index of file/color cannot be found"
			return
		assert c0i==row
		
		new_name = name_dialog(old_name)
		if new_name==None:
			return
		self.names[c0i] = new_name
		self.__build_list()
		self.__do_config()
	
	def __change_color(self, row):
		c0 = self.list.get_text(row, 0)
		try:
			c0i = self.watches.index(c0)
		except:
			print "[MultiTail] index of file/color cannot be found"
			return
		assert c0i==row
		
		color = color_dialog()
		if color == None:
			return
		
		self.colors[c0i] = color
		self.__build_list()
		self.__do_config()
		
	def __add_file(self, button):
		filename = file_dialog()
		if filename==None:
			return
		if filename in self.watches:
			dialog.warning("Duplicate file", "Duplicate file")
			return
		
		self.watches.append(filename)
		self.names.append(os.path.basename(filename))
		self.colors.append("#000000")
		self.states.append("ok")
		assert len(self.watches)==len(self.names)==len(self.colors)==len(self.states)
		
		self.__build_list()
		self.__do_config()
	
	def __reset_file(self, button):
		if not self.__selected_row==-1:
			row = self.__selected_row
			self.states[row]="ok"
			#self.__selected_row=-1
			self.__build_list()
			self.__do_config()
		
	def __remove_file(self, button):
		if not self.__selected_row==-1:
			row = self.__selected_row
			del self.watches[row]
			del self.names[row]
			del self.colors[row]
			del self.states[row]
			assert len(self.watches)==len(self.colors)==len(self.states)==len(self.names)
			#self.__selected_row=-1
			self.__build_list()
			self.__do_config()

def show(get_config, set_config):
	global win
	assert win==None, "[MultiTail] listdialog.show called while window was still opened"
	win = WatchList(get_config, set_config)

def file_dialog():
	d = gtk.FileSelection()
	result = d.run()
	filename = d.get_filename()
	d.destroy()
	if result==gtk.RESPONSE_OK and os.path.isfile(filename):
	   	return filename
	else:
		return None

def name_dialog(old_name=""):
	d = gtk.Dialog(title="Name the tail", buttons=("Ok", 1, "Cancel", 0))
	e = gtk.Entry()
	e.set_text(old_name)
	d.vbox.pack_start(e)
	d.show_all()
	result = d.run()
	new_name = e.get_text()
	d.destroy()
	if result==1:
		return new_name
	else:
		return None

def color_dialog():
	d = gtk.ColorSelectionDialog("Choose color")
	colorsel = d.colorsel
	colorsel.set_has_palette(gtk.TRUE)
	result = d.run()
	c = colorsel.get_current_color()
	red = ("00" + hex(c.red / 256)[2:])[-2:]
	green = ("00" + hex(c.green / 256)[2:])[-2:]
	blue = ("00" + hex(c.blue / 256)[2:])[-2:]
	d.destroy()
	if result==gtk.RESPONSE_OK:	
		return "#%s%s%s" % (red, green, blue)
	else:
		return None
