from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n
import time
import os

class VariableBorder(Sensor):

	def __init__(self):
		
		global _; _ = i18n.Translator("variable-border-sensor")
		Sensor.__init__(self)
		
		self._set_config_type("height", TYPE_INT, 10)
		self._set_config_type("top-visible", TYPE_BOOL, 1)
		self._set_config_type("bottom-visible", TYPE_BOOL, 1)
		self._set_config_type("separator", TYPE_BOOL, 1)
		
		self._add_timer(500, self.__conf_change)
		self._watch_config(self.config_watcher)
	
	def config_watcher(self,key,value):
		self.__conf_change()
		
	def __conf_change(self):
		data = self._new_output()
		
		h = self._get_config("height")
		top_vis = self._get_config("top-visible")
		bot_vis = self._get_config("bottom-visible")
		
		disp_h = h
		cen_y = 0
		
		if top_vis:
			disp_h += 17
			cen_y = 17
			
		if bot_vis:
			disp_h += 25
		
		data.set("top-visible", top_vis)
		data.set("bottom-visible", bot_vis)
		data.set("central-height", h)
		data.set("central-y", cen_y)
		data.set("bottom-y", cen_y + h)
		data.set("display-height", disp_h)
		data.set("separator-visible", self._get_config("separator"))
		
		self._send_output(data)
		
		return 0

	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name(_("Border"))
		configurator.add_title(_("Height"))
		configurator.add_spin(_("Height:"), "height",
					_("Changes the border height"),
					1, 2000)
		configurator.add_checkbox(_("Top"), "top-visible",
					_("Shows or hides the top border"))
		configurator.add_checkbox(_("Bottom"), "bottom-visible",
					_("Shows or hides the bottom border"))
		configurator.add_checkbox(_("Separator"), "separator",
					_("Shows or hides the central separator"))
		return configurator

def new_sensor(args): return apply(VariableBorder, args)
