from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import libdesklets as lib
from libdesklets import system

import os
import re
import math

class WirelessSensor( Sensor ):
	def __init__( self ):
		self.__buffer = ""
		self.__status = ""
		
		global _; _ = i18n.Translator( "wireless-sensor" )
		Sensor.__init__( self )
		self._add_timer( 5000, self.__update )
		
		# setup configurables		
		self._set_config_type( "update", TYPE_INT, 5000 )
		self._set_config_type( "device", TYPE_STRING, "eth0" )
		self._set_config_type( "bar_color", TYPE_STRING, "#336699" )
		self._set_config_type( "bg_color", TYPE_STRING, "#000000" )
		
	#
	# setup configuration dialog
	#		
	def get_configurator( self ):
	
		# list available net devices
		net = []
		for dev in lib.system.net_devices():
			net.append( [dev, dev] )
		
		# setup configurator		
		configurator = self._new_configurator()
		configurator.set_name( "Wireless" )
		configurator.add_title( _("Settings") )
		configurator.add_spin( _("Update every (ms):"), "update",
							  "Updates the display every x milliseconds",
							  1000, 10000 )
		configurator.add_option( _("Device:"), "device",
							    "Specify network device", net )
		configurator.add_color_selector( _("Bar Color:"), "bar_color",
										"Color of the bar chart foreground" )
		configurator.add_color_selector( _("Background Color:"), "bg_color",
										"Color of the bar chart background" )
		return configurator

	#
	# grab config info, link status, and pass it on to the display
	#	
	def __update( self ):
		# get the up to date config information
		device = self._get_config( "device" )
		update = self._get_config( "update" )
		bar_color = self._get_config( "bar_color" )
		bg_color = self._get_config( "bg_color" )
		
		# get link status for device from iwconfig(8)
		cmd = "/sbin/iwconfig " + device
		iwconfig = os.popen( cmd, 'r' );
		self.buffer = iwconfig.read()
		self.__parse()
		
		# output link status and config options to display
		output = self._new_output()
		output.set( "status", self.__status )
		output.set( "device", device )
		output.set( "bar_color", bar_color )
		output.set( "bg_color", bg_color )
		self._send_output( output )
		
		# run again at configured update interval
		self._add_timer( update, self.__update )
		# don't return 1 or True so interval is configurable
		
	#
	# Take output of iwconfig(8) and turn it into a percentage
	#	
	def __parse( self ):
		result = re.compile( r'Link Quality:([0-9]{1,2})' ).search( self.buffer )
		if (result != None):
				quality = float( result.group( 1 ) )
				percent = int( math.log( quality ) / math.log( 92 ) * 100 )
				self.__status = percent
		else:
				self.__status = 0

def new_sensor( args ): return apply( WirelessSensor, args )
