## XMLQuotes - A DeskQuote version with XML support
#   - Created by: Daniel Alves <daniel.alves@linux-portugal.org>

## 2.4

from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n
from xmldata import XMLData
from quotewindows import QuotesEditor, CreateFileWindow

import os
import commands
import time

# XMLQuotes, a sensor package to put random quotes on your desktop
class XMLQuotes(Sensor):

	def __init__(self, sourcefile, textalignment, interval, width):
		global _; _ = i18n.Translator("xmlquotes-sensor")
	        # instance variable for counting
        	self.__quote = ''
		self.__interval = 0
		self.__need_run = 1
		self.file_exists = 0
		self.__xmldata = XMLData(sourcefile)

	        # call the constructor of the super class
        	Sensor.__init__(self)
        	
        	# Create config types
		self._set_config_type("update-frequency", TYPE_INT, interval)
		self._set_config_type("update-width", TYPE_INT, int(width))
		self._set_config_type("quote-file", TYPE_STRING, sourcefile)
		self._set_config_type("text-alignment", TYPE_STRING, textalignment)
		self._add_timer(self.__interval, self.__on_timer)

	def get_configurator(self):
	        configurator = self._new_configurator()
        	configurator.set_name(_("Panel"))
        	configurator.add_title(_("Appearance"))
		configurator.add_spin(_("Refresh:"), "update-frequency",_("The update rate in seconds"),10, 1200)
		configurator.add_spin(_("Width:"), "update-width",_("The approximate width in characters"),20, 60)
		configurator.add_title(_("Quote File"))
		configurator.add_entry(_("Quote File"), "quote-file", _("The file to read quotes from"))
		configurator.add_title(_("Text Alignment"))
		configurator.add_option(_("Text Alignment"), "text-alignment",
                                _("Left, Right or Center alignment"),
                                [(_("Left"), "left"),
                                 (_("Center"), "center"), (_("Right"), "right")])
		
		return configurator

# Call action (from the display)
	def call_action(self, action, path, args = []):
		if (action == "click"):
			if ( self.file_exists == 0 ): createwnd = CreateFileWindow( self._get_config("quote-file"), self.file_created )
			else: self.__editor;
			
		if (action == "open_menu"):
			if ( self.file_exists != 0 ): self.open_menu ( [ (_ ( "Quotes Editor" ), 1, None, self.__editor, [] ) ] )

# File creation callback
	def file_created(self, sourcefile):
		self._set_config("quote-file", sourcefile)
		self.__on_timer()

# Quote editor callback
	def __editor(self, args):
		qedit = QuotesEditor( self._get_config("quote-file") )

# A function to align text, from Magnus L. Hetland (found on usenet)
	def __align(self, line, width, alignment):
    		space = width - len(line)
    		if alignment == "center":
        		return ' '*(space/2) + line + \
               		' '*(space/2 + space%2)
    		elif alignment == "right":
        		return ' '*space + line
    		else:
        		return line + ' '*space

# On Timer function (Main function)
	def __on_timer(self):
		# Kill timer if interval was changed
		if ( self.__interval != self._get_config("update-frequency") and self.__need_run == 0 ):
			self.__interval = int( self._get_config("update-frequency") )
			self._add_timer( self.__interval*1000, self.__on_timer )
			return 0
		# Continuing
		quote = ""
		aligned = ""
		self.__need_run = 0
		# Get text width and quotes file
		linewidth = self._get_config("update-width")
		sourcefile = self._get_config("quote-file")
		# Check file source
		if self.__xmldata.get_file() != sourcefile:
			self.__xmldata.set_file(sourcefile)
		# Check if file exists
		get_xml = 1
		try:
			open(sourcefile)
		except:
			self.file_exists = 0
			prequote = "The quote file does not exist.\nClick here to create it"
			quote_author = ""
			get_xml = 0
		# Get quote and author
		if ( get_xml == 1 ):
			self.file_exists = 1
			self.__xmldata.open_dom()
			prequote = self.__xmldata.get_quote()
			quote_author = self.__xmldata.get_author()
			self.__xmldata.close_dom()
		# clean up quote
		quotelines = prequote.split("\n")
		for quoteline in quotelines:
			s = quoteline.strip()
			if (len(s) >= linewidth):
				length = len(s)
				string1 = s[0:linewidth]
				string2 = s[linewidth:length]
				spacer = linewidth - 1
				while ( string1[spacer] != ' ' ): spacer = spacer - 1
				# Feed the shortened strings into the quote
				quote = quote + string1[0:spacer+1] + "\n" + string1[spacer+1:linewidth] + string2 + "\n"
			else: quote = quote + s + "\n"
		quote = quote.strip()
		# end_for
		if not quote_author == "":
			quote = quote + "\n\t\n - " + quote_author
		# align the text
		aligned = self.__align( quote, self._get_config("update-width"), self._get_config("text-alignment") )
		# Now we're ready to pass the data out to the display
		data = self._new_output()
		data.set("quote", aligned)
		self._send_output(data)
		# Return true
		return 1

def new_sensor(args): return apply(XMLQuotes, args)
