## XMLQuotes - editor.py
#   - Created by: Daniel Alves <daniel.alves@linux-portugal.org>

## 2.4

from xmldata import XMLData

import os
import commands
import gtk
import gobject

class MsgBox:
	def __init__(self, msg, title):
		# Create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.destroy)
		self.window.set_border_width(10)
		self.window.set_title(title)
		# Create boxes
		self.main_vbox = gtk.VBox(gtk.FALSE, 5)
		self.window.add(self.main_vbox)
		hbox_msg = gtk.HBox(gtk.FALSE)
		hbox_buttons = gtk.HBox(gtk.FALSE)
		# Create controls
		label_msg = gtk.Label(msg)
		button_ok = gtk.Button("OK")
		button_ok.connect("clicked", self.click_button_ok)
		hbox_msg.pack_start(label_msg, gtk.FALSE, gtk.FALSE, 5)
		hbox_buttons.pack_start(button_ok, gtk.FALSE, gtk.FALSE, 5)
		# Pack all
		self.main_vbox.pack_start(hbox_msg, gtk.FALSE, gtk.FALSE)
		self.main_vbox.pack_start(hbox_buttons, gtk.FALSE, gtk.FALSE)
		# Start
		self.window.show_all()
		gtk.main()

	def destroy(self):
		self.window.hide()
		gtk.main_quit()

	def click_button_ok(self, *args):
		self.window.destroy()

class QuotesEditor:
	def __init__(self, xmlfile):
		self.selectedID = -1
		self.xml = XMLData(xmlfile)
		self.xml.open_dom()
		# Create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.destroy)
		self.window.set_border_width(5)
		self.window.set_title("Quotes Editor")
		# Create boxes
		main_vbox = gtk.VBox(gtk.FALSE, 5)
		self.window.add(main_vbox)
		hbox_sw = gtk.HBox(gtk.FALSE)
		hbox_buttons = gtk.HBox(gtk.FALSE)
		# Create scrolled treeview
		sw = gtk.ScrolledWindow()
		sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
		self.model = gtk.TreeStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_PYOBJECT, gtk.gdk.Pixbuf)
		treeview = gtk.TreeView(self.model)
		treeview.set_size_request(600, 400)
		sw.add(treeview)
		## render columns
		renderer = gtk.CellRendererText()
		column_author = gtk.TreeViewColumn("Author", renderer, text=0)
		column_quote = gtk.TreeViewColumn("Quote", renderer, text=1)
		treeview.append_column(column_author)
		treeview.append_column(column_quote)
		selection = treeview.get_selection()
		selection.connect("changed", self.change_selection, None)
		hbox_sw.pack_start(sw, gtk.FALSE, gtk.FALSE)
		# Create buttons
		button_add = gtk.Button("Add Quote")
		button_add.connect("clicked", self.click_button_add)
		button_edit = gtk.Button("Edit Quote")
		button_edit.connect("clicked", self.click_button_edit)
		button_remove = gtk.Button("Remove Quote")
		button_remove.connect("clicked", self.click_button_remove)
		button_quit = gtk.Button("Quit")
		button_quit.connect("clicked", self.click_button_quit)
		## Add buttons
		hbox_buttons.pack_start(button_add, gtk.FALSE, gtk.FALSE, 0)
		hbox_buttons.pack_start(button_edit, gtk.FALSE, gtk.FALSE, 5)
		hbox_buttons.pack_start(button_remove, gtk.FALSE, gtk.FALSE, 5)
		hbox_buttons.pack_end(button_quit, gtk.FALSE, gtk.FALSE, 0)
		# Pack all in VBox
		main_vbox.pack_start(hbox_sw, gtk.FALSE, gtk.FALSE)
		main_vbox.pack_start(hbox_buttons, gtk.FALSE, gtk.FALSE)
		# Shows all
		self.window.show_all()
		self.fill()
		gtk.main()

	## other methods
	def fill(self):
		self.model.clear()
		self.xml.get_all_quotes(self.add_quote)

	def add_quote(self, quote_id, quote, author):
		new = self.model.append(None, None)
		self.model.set_value( new, 0, author.strip() )
		self.model.set_value( new, 1, quote.strip() )
		self.model.set_value(new, 2, quote_id)

	## Event handlers
	def change_selection(self, selection, data=None):
		mod, iter = selection.get_selected()
		test_iter = mod.append(None, None)
		if type(iter) == type(test_iter):
			self.selectedID = mod.get_value(iter, 2)
		else: self.selectedID = -1

	def click_button_add(self, *args):
		add_window = AddWindow(self)

	def click_button_edit(self, *args):
		if self.selectedID != -1:
			edit_window = EditWindow(self, self.selectedID)

	def click_button_remove(self, *args):
		if self.selectedID != -1:
			self.xml.remove_quote(self.selectedID)
		self.fill()

	def click_button_quit(self, *args):
		self.window.destroy()

	def destroy(self, *args):
		self.xml.close_dom()
		self.window.hide()
		gtk.main_quit()

class CreateFileWindow:
	def __init__(self, sourcefile, callback):
		self.__callback = callback
		# Create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.destroy)
		self.window.set_border_width(5)
		self.window.set_title("Create Quotes file")
		# Create boxes
		self.main_vbox = gtk.VBox(gtk.FALSE, 5)
		self.window.add(self.main_vbox)
		hbox_warning = gtk.HBox(gtk.FALSE)
		hbox_file = gtk.HBox(gtk.FALSE)
		hbox_buttons = gtk.HBox(gtk.FALSE)
		# Create labels and edits
		label_warning = gtk.Label("XMLDeskQuotes is configured to get the quotes from the following xml file.\nHowever, the file does not exist. Would you like to create it?")
		label_file = gtk.Label("Source file:")
		self.edit_file = gtk.TextView()
		self.edit_file.set_size_request(400, 20)
		buffer_file = self.edit_file.get_buffer()
		buffer_file.set_text(sourcefile)
		hbox_warning.pack_start(label_warning, gtk.FALSE, gtk.FALSE, 5)
		hbox_file.pack_start(label_file, gtk.FALSE, gtk.FALSE, 5)
		hbox_file.pack_start(self.edit_file, gtk.FALSE, gtk.FALSE, 5)
		# Create buttons
		button_create = gtk.Button("Create")
		button_create.connect("clicked", self.click_button_create)
		button_cancel = gtk.Button("Cancel")
		button_cancel.connect("clicked", self.click_button_cancel)
		hbox_buttons.pack_start(button_create, gtk.FALSE, gtk.FALSE, 5)
		hbox_buttons.pack_end(button_cancel, gtk.FALSE, gtk.FALSE, 5)
		# Pack all
		self.main_vbox.pack_start(hbox_warning, gtk.FALSE, gtk.FALSE)
		self.main_vbox.pack_start(hbox_file, gtk.FALSE, gtk.FALSE)
		self.main_vbox.pack_start(hbox_buttons, gtk.FALSE, gtk.FALSE)
		self.window.show_all()
		gtk.main()
		
	def destroy(self, *args):
		self.window.hide()
		gtk.main_quit()

	def click_button_create(self, *args):
		buffer_file = self.edit_file.get_buffer()
		startiter_file = buffer_file.get_start_iter()
		quotesfile = startiter_file.get_text( buffer_file.get_end_iter() )
		command = "cp " + os.getcwd() + "/Sensors/XMLQuotes/quotes.xml " + quotesfile
		try:
			print os.system(command)
			self.__callback(quotesfile)
		except:
			wmsgbox = MsgBox("Error creating file", "Error")
		# Exit window
		self.window.destroy()

	def click_button_cancel(self, *args):
		self.window.destroy()

class TemplateWindow:
	def __init__(self):
		# Create window
		self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
		self.window.connect("destroy", self.destroy)
		self.window.set_border_width(10)
		self.window.set_title("Add Quote")
		# Create boxes
		self.main_vbox = gtk.VBox(gtk.FALSE, 5)
		self.window.add(self.main_vbox)
		hbox_author = gtk.HBox(gtk.FALSE)
		hbox_quote = gtk.HBox(gtk.FALSE)
		# Create labels and edits
		label_author = gtk.Label("Author")
		label_quote = gtk.Label("Quote")
		self.edit_author = gtk.TextView()
		self.edit_quote = gtk.TextView()
		self.edit_author.set_size_request(200, 15)
		self.edit_quote.set_size_request(400, 100)
		hbox_author.pack_start(label_author, gtk.FALSE, gtk.FALSE, 5)
		hbox_author.pack_start(self.edit_author, gtk.FALSE, gtk.FALSE, 5)
		hbox_quote.pack_start(label_quote, gtk.FALSE, gtk.FALSE, 5)
		hbox_quote.pack_start(self.edit_quote, gtk.FALSE, gtk.FALSE, 5)
		# Pack all in VBox
		self.main_vbox.pack_start(hbox_author, gtk.FALSE, gtk.FALSE)
		self.main_vbox.pack_start(hbox_quote, gtk.FALSE, gtk.FALSE)

	def start(self):
		self.window.show_all()
		gtk.main()
		
	def destroy(self, *args):
		self.window.hide()
		gtk.main_quit()

class AddWindow(TemplateWindow):
	def __init__(self, caller):
		self.__caller = caller
		# Create window
		TemplateWindow.__init__(self)
		# Create box
		hbox_buttons = gtk.HBox(gtk.FALSE)
		# Create buttons
		button_add = gtk.Button("Add")
		button_add.connect("clicked", self.click_button_add)
		button_cancel = gtk.Button("Cancel")
		button_cancel.connect("clicked", self.click_button_cancel)
		# Pack all
		hbox_buttons.pack_start(button_add, gtk.FALSE, gtk.FALSE, 5)
		hbox_buttons.pack_end(button_cancel, gtk.FALSE, gtk.FALSE, 5)
		self.main_vbox.pack_start(hbox_buttons, gtk.FALSE, gtk.FALSE)
		# Start all
		self.start()

	## Event handlers
	def click_button_add(self, *args):
		buffer_author = self.edit_author.get_buffer()
		startiter_author = buffer_author.get_start_iter()
		enditer_author = buffer_author.get_end_iter()
		author = startiter_author.get_text(enditer_author)

		buffer_quote = self.edit_quote.get_buffer()
		startiter_quote = buffer_quote.get_start_iter()
		enditer_quote = buffer_quote.get_end_iter()
		quote = startiter_quote.get_text(enditer_quote)

		self.__caller.xml.add_quote(quote, author)
		self.__caller.fill()
		# destroy self
		self.window.destroy()

	def click_button_cancel(self, *args):
		self.window.destroy()

class EditWindow(TemplateWindow):
	def __init__(self, caller, quote_id):
		self.__caller = caller
		self.__quote_id = quote_id
		# Create window
		TemplateWindow.__init__(self)
		# Get values
		quote = caller.xml.get_quote_by_id(quote_id).strip()
		author = caller.xml.get_author().strip()
		# Add values to edit
		buffer_author = self.edit_author.get_buffer()
		buffer_quote = self.edit_quote.get_buffer()
		if author != "":
			buffer_author.set_text(author)
		if quote != "":
			buffer_quote.set_text(quote)
		# Create box
		hbox_buttons = gtk.HBox(gtk.FALSE)
		# Create buttons
		button_edit = gtk.Button("Edit")
		button_edit.connect("clicked", self.click_button_edit)
		button_cancel = gtk.Button("Cancel")
		button_cancel.connect("clicked", self.click_button_cancel)
		# Pack all
		hbox_buttons.pack_start(button_edit, gtk.FALSE, gtk.FALSE, 5)
		hbox_buttons.pack_end(button_cancel, gtk.FALSE, gtk.FALSE, 5)
		self.main_vbox.pack_start(hbox_buttons, gtk.FALSE, gtk.FALSE)
		# Start all
		self.start()

	## Event handlers
	def click_button_edit(self, *args):
		buffer_author = self.edit_author.get_buffer()
		startiter_author = buffer_author.get_start_iter()
		enditer_author = buffer_author.get_end_iter()
		author = startiter_author.get_text(enditer_author)

		buffer_quote = self.edit_quote.get_buffer()
		startiter_quote = buffer_quote.get_start_iter()
		enditer_quote = buffer_quote.get_end_iter()
		quote = startiter_quote.get_text(enditer_quote)

		self.__caller.xml.set_quote(self.__quote_id, quote, author)
		self.__caller.fill()
		# destroy self
		self.window.destroy()

	def click_button_cancel(self, *args):
		self.window.destroy()
