from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import os
from os.path import getsize
import time
import mailbox
import poplib
import imaplib

def human_readable(value):
        if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value


class enhallmail(Sensor):
	def __init__(self, interval = '1000', popboxes = 1):
		self.__newmail = 0
		self.__msgcount = 0
		self.__msgnews = 0
		self.__boxsize = 0
		#self.__mailtext = ""
		self.__popboxes = popboxes
		
		Sensor.__init__(self)
		
		self._set_config_type('update', TYPE_INT, int(interval))
		self._set_config_type('maillabel', TYPE_STRING, '')
		
		self._set_config_type('mbox_show', TYPE_BOOL, 0)
		self._set_config_type('mailboxmbox', TYPE_STRING, '')
		
		self._set_config_type('qbox_show', TYPE_BOOL, 0)
		self._set_config_type('mailboxqbox', TYPE_STRING, '')
		
		self._set_config_type('pop_show1', TYPE_BOOL, 0)
		self._set_config_type('pop_server1', TYPE_STRING, '')
		self._set_config_type('pop_login1', TYPE_STRING, '')
		self._set_config_type('pop_password1', TYPE_SECRET_STRING, '')
		self._set_config_type('pop_show2', TYPE_BOOL, 0)
		self._set_config_type('pop_server2', TYPE_STRING, '')
		self._set_config_type('pop_login2', TYPE_STRING, '')
		self._set_config_type('pop_password2', TYPE_SECRET_STRING, '')
		self._set_config_type('pop_show3', TYPE_BOOL, 0)
		self._set_config_type('pop_server3', TYPE_STRING, '')
		self._set_config_type('pop_login3', TYPE_STRING, '')
		self._set_config_type('pop_password3', TYPE_SECRET_STRING, '')
		self._set_config_type('imap_show1', TYPE_BOOL, 0)
		self._set_config_type('imap_server1', TYPE_STRING, '')
		self._set_config_type('imap_login1', TYPE_STRING, '')
		self._set_config_type('imap_password1', TYPE_SECRET_STRING, '')
		self._set_config_type('imap_folder1', TYPE_STRING, '')
		self._set_config_type('imap_multifolders1', TYPE_BOOL, 0)
		self._set_config_type('notify_enabled',TYPE_BOOL, 0)
		self._set_config_type('notify_command',TYPE_STRING, '')
		
		"""
		self._add_timer(int(interval), self.__on_tick)
		"""
		self._add_thread(self.__command_thread, 0.001 * int(interval))


	def get_configurator(self):
		configurator = self._new_configurator()
		configurator.set_name('Mailbox check')
		configurator.add_title('Display Label')
		configurator.add_entry('Label', 'maillabel', 'Label in display')

		configurator.add_title('Mailbox Settings')
		configurator.add_spin('Update every (ms):', 'update',
                                      'Updates the display(s) every x milliseconds',
                                      1*1000, 3600*24*1000)
				      
		configurator.add_title('Unix Mailbox Settings (mbox style)')
		configurator.add_checkbox("Enable","mbox_show","Check this box")
		configurator.add_entry('Mailbox', 'mailboxmbox', 'Mailbox path')
		
		configurator.add_title('Qmail (.maildir) Settings')
		configurator.add_checkbox("Enable","qbox_show","Check this box")
		configurator.add_entry('Mailbox', 'mailboxqbox', 'Mailbox path')
		
		if self.__popboxes >= 1:
			configurator.add_title('Pop Mailbox Settings')
			configurator.add_checkbox("Enable","pop_show1","Check this box")
			configurator.add_entry("Host 1","pop_server1","Host (pop server)")
			configurator.add_entry("Login 1","pop_login1","Login")
			configurator.add_entry("Password 1","pop_password1","Password",passwd=1)
		if self.__popboxes >= 2:
			configurator.add_title('Pop Mailbox 2 Settings')
			configurator.add_checkbox("Enable","pop_show2","Check this box")
			configurator.add_entry("Host 2","pop_server2","Host (pop server)")
			configurator.add_entry("Login 2","pop_login2","Login")
			configurator.add_entry("Password 2","pop_password2","Password",passwd=1)
		if self.__popboxes >= 3:
			configurator.add_title('Pop Mailbox 3 Settings')
			configurator.add_checkbox("Enable","pop_show3","Check this box")
			configurator.add_entry("Host 3","pop_server3","Host (pop server)")
			configurator.add_entry("Login 3","pop_login3","Login")
			configurator.add_entry("Password 3","pop_password3","Password",passwd=1)
			
		configurator.add_title('IMAP Mailbox Settings')
		configurator.add_checkbox("Enable","imap_show1","Check this box")
		configurator.add_entry("IMAP Host","imap_server1","Host (imap server)")
		configurator.add_entry("Login","imap_login1","Login")
		configurator.add_entry("Password","imap_password1","Password",passwd=1)
		configurator.add_entry("Folder(s)","imap_folder1","IMAP-Folder(s) to check")
		configurator.add_checkbox("Enable multiple folders check","imap_multifolders1","If checked, more than one folder can be entered above, seperated by '&&'")
		configurator.add_title('Notification')
		configurator.add_checkbox("External command","notify_enabled","Run an external command when new mail is available")
		configurator.add_entry("Command","notify_command","Run this command each time the number of new mails change")
		return configurator


	def __on_tick(self):
		data = self._new_output()
		data.set('newmail', self.__newmail)
		data.set('msgcount',self.__msgcount)
		data.set('msgnews',self.__msgnews)
		if self.__boxsize != -1 :
			if self.__boxsize != 'Connection error':
				data.set('boxsize', '(' + human_readable(self.__boxsize) + 'B)')
			else :
				data.set('boxsize', '(' + self.__boxsize + ')')
			
			if self.__boxsize == 'Connection error':
				data.set('error', 1)
			else :
				data.set('error', 0)
		else :
			data.set('error', 0)
			
			
		if self.__newmail == 0 and self.__boxsize != 'Connection error':
			data.set('nonewmail', 1)
		else :
			data.set('nonewmail', 0)
			
		data.set('maillabel', self._get_config('maillabel'))
		data.set('mailboxmbox', self._get_config('mailboxmbox'))
		data.set('mailboxqbox', self._get_config('mailboxqbox'))
		data.set('host1', self._get_config('pop_server1'))
		data.set('login1', self._get_config('pop_login1'))
		data.set('host2', self._get_config('pop_server2'))
		data.set('login2', self._get_config('pop_login2'))
		data.set('host3', self._get_config('pop_server3'))
		data.set('login3', self._get_config('pop_login3'))
		data.set('imaphost1', self._get_config('imap_server1'))
		data.set('imaplogin1', self._get_config('imap_login1'))
		#data.set('mailtext', self.__mailtext)

		self._send_output(data)
		return 1


	def __command_thread(self, interval):
		while (1):
			if (self._is_stopped()):
                                break
                        self.__msgcount = 0
                        self.__msgnews = 0
			self.__boxsize = 0
			self.__error = 0
			oldmailstatus = self.__newmail
			mailtext = ""
			try:
				if (self._get_config('mbox_show')):
                                	mailboxpath = self._get_config('mailboxmbox')
                                	self.__boxsize += getsize(mailboxpath)
					mb = mailbox.PortableUnixMailbox(file(mailboxpath))
					for m in mb:
						self.__msgcount += 1
						status = m.get('status', 'N')
						if status.find('N') != -1:
							self.__msgnews += 1
					
					#if self.__msgnews>0:
					#	mailtext = "\n"+str(self.__msgnew)+" messages in "+mailboxpath

				if (self._get_config('qbox_show')):
                                	mailboxpath = self._get_config('mailboxqbox')
                                	self.__boxsize += getsize(mailboxpath)
					mb = mailbox.Maildir(mailboxpath)
					i = 0
					j = 0
					for q in os.listdir(mailboxpath+'/cur'):
						i += 1
					for p in os.listdir(mailboxpath+'/new'):
						j += 1
					self.__msgcount += i
					self.__msgnews += j
					print self.__msgnews
					
					#if j>0:
					#	mailtext = "\n"+str(j)+" messages in "+mailboxpath

				if (self._get_config('pop_show1')):
					pop = poplib.POP3(self._get_config("pop_server1"))
					pop.user(self._get_config("pop_login1"))
					pop.pass_(self._get_config("pop_password1"))

					i = pop.stat()[0]
					j = pop.stat()[0]
					self.__msgcount += i
					self.__msgnews  += j
					self.__boxsize += pop.stat()[1]
					
					pop.quit()

					#if j>0:
					#	mailtext = "\n"+str(j)+" messages on "+self._get_config("pop_server1")
					
				if (self._get_config('pop_show2')):
					pop = poplib.POP3(self._get_config("pop_server2"))
					pop.user(self._get_config("pop_login2"))
					pop.pass_(self._get_config("pop_password2"))
					i = pop.stat()[0]
					j = pop.stat()[0]
					self.__msgcount += i
					self.__msgnews  += j
					self.__boxsize += pop.stat()[1]
					
					pop.quit()

					#if j>0:
					#	mailtext = "\n"+str(j)+" messages on "+self._get_config("pop_server1")

				if (self._get_config('pop_show3')):
					pop = poplib.POP3(self._get_config("pop_server3"))
					pop.user(self._get_config("pop_login3"))
					pop.pass_(self._get_config("pop_password3"))
					i = pop.stat()[0]
					j = pop.stat()[0]
					self.__msgcount += i
					self.__msgnews  += j
					self.__boxsize += pop.stat()[1]
					
					pop.quit()

					#if j>0:
					#	mailtext = "\n"+str(j)+" messages on "+self._get_config("pop_server1")

				if (self._get_config('imap_show1')):
					imap = imaplib.IMAP4(self._get_config("imap_server1"))
					imap.login(self._get_config("imap_login1"),self._get_config("imap_password1"))
					
					if self._get_config("imap_multifolders1"):
						boxes = self._get_config("imap_folder1").split("&&")
					else:
						boxes = [ self._get_config("imap_folder1") ]
					for box in boxes:
						box = box.strip()
						imap.select(box,readonly=1)
						imap.noop()

						unseen = imap.search(None, '(UNSEEN UNDELETED)')
						total = imap.search(None, '(UNDELETED)')
						#self.__msgcount += pop.stat()[0]
						#self.__msgnews  += pop.stat()[0]
						#self.__boxsize += pop.stat()[1]
						i = len(total[1][0].split())
						j = len(unseen[1][0].split())
						self.__msgcount += i
						self.__msgnews += j
						self.__boxsize = -1
						#if j>0:
						#	mailtext = "\n"+str(j)+" messages in "+box+" on "+self._get_config("imap_server1")

						imap.close()
						
					imap.logout()
			except:
				self.__msgcount = 0
				self.__msgnews = 0
				self.__boxsize = 'Connection error'
			if self.__msgnews : 
				self.__newmail = 1
				if oldmailstatus == 0 :
					cmd = self._get_config("notify_command")
					os.system("cd ~ && " + cmd + " & disown")
				#self.__mailtext = mailtext

			else :
				self.__newmail = 0
				

			self.__on_tick()
			time.sleep(0.001 * self._get_config('update'))
				

def new_sensor(args):
        return apply(enhallmail, args)
