from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import commands
import os
import time
import poplib

def human_readable(value):
	if value >= (1024**4):
		return "%.2fT" % (value/(1024.0**4))
	if value >= (1024**3):
		return "%.2fG" % (value/(1024.0**3))
	if value >= (1024**2):
		return "%.2fM" % (value/(1024.0**2))
	if value >= 1000:
		return "%.2fK" % (value/1024.0)
	else : return "%i" % value

class popmail(Sensor):
	
	def __init__(self, interval = "60000"):
		self.__newmail = 0
		self.__msgcount = 0
		self.__boxsize = 0
		
		Sensor.__init__(self)
		
		self._set_config_type("update", TYPE_INT, int(interval))
		self._set_config_type("pop_server", TYPE_STRING, "")
		self._set_config_type("pop_login", TYPE_STRING, "")
		self._set_config_type("pop_password", TYPE_SECRET_STRING, "")
		
		self._add_timer(1000, self.__on_tick)
		self._add_thread(self.__command_thread, 0.001 * int(interval))

	def get_configurator(self):

		configurator = self._new_configurator()
		configurator.set_name("Pop mail check")
		configurator.add_title("Settings")
		configurator.add_spin("Update every (ms):", "update",
					"Updates the display(s) every x milliseconds",
					1*1000, 3600*24*1000)
		configurator.add_entry("Host", "pop_server",
					"Host (pop server)")
		configurator.add_entry("Login", "pop_login",
					"Login")
		configurator.add_entry("Password", "pop_password",
					"Password",passwd = 1)
		
		return configurator

	def __on_tick(self):

		data = self._new_output()
		data.set("newmail", self.__newmail)
		data.set("msgcount",self.__msgcount)
		if self.__boxsize != "Connection error":
			data.set("boxsize",human_readable(self.__boxsize) + "B")
		else :
			data.set("boxsize",self.__boxsize)
			
		if self.__boxsize=="Connection error":
			data.set("error",1)
		else :
			data.set("error",0)
			
		if self.__newmail==0 and self.__boxsize!="Connection error":
			data.set("nonewmail", 1)
		else :
			data.set("nonewmail", 0)
			
		data.set("host", self._get_config("pop_server"))
		data.set("login", self._get_config("pop_login"))
		self._send_output(data)

		return 1



	def __command_thread(self, interval):

		while (1):
			if (self._is_stopped()): break
			
			try :
				pop = poplib.POP3(self._get_config("pop_server"))
				pop.user(self._get_config("pop_login"))
				pop.pass_(self._get_config("pop_password"))
			
				self.__msgcount,self.__boxsize = pop.stat()
			
				pop.quit()
				
			except :
				self.__msgcount = 0
				self.__boxsize = "Connection error"
			
			
			if self.__msgcount : 
				self.__newmail = 1
			else :
				self.__newmail = 0
			
			
			time.sleep(0.001*self._get_config("update"))
				


def new_sensor(args): return apply(popmail, args)